"""Visualize scalar values interactively
by hovering the mouse on a mesh
Press c to clear the path"""
from vedo import *

def func(evt):                       ### called every time mouse moves!
    msh = evt.actor
    if not msh: return               # mouse hits nothing, return.
    pt = evt.picked3d                # 3d coords of point under mouse

    pid = msh.closestPoint(pt, returnPointId=True)
    txt = f"Point:  {precision(pt[:2]  ,2)}\n" \
          f"Height: {precision(arr[pid],3)}\n" \
          f"Ground speed: {precision(evt.speed3d*100,2)}"
    arw = Arrow(pt - evt.delta3d, pt, s=0.001, c='orange5')
    vig = msh.vignette(txt, point=pt, offset=(0.4,0.6),
                       s=0.04, c='k', font="VictorMono").followCamera()
    msg.text(txt)                    # update text message
    if len(plt.actors)>3: plt.pop()  # remove the old vignette
    plt.add([arw, vig])              # add Arrow and the new vignette

msg = Text2D(pos='bottom-left', font="VictorMono") # an empty text
hil = ParametricShape('RandomHills').cmap('terrain').addScalarBar()
arr = hil.pointdata["Scalars"]       # numpy array with heights

plt = Plotter(axes=1, bg2='lightblue')
plt.addCallback('mouse move', func)  # add the callback function
plt.addCallback('keyboard', lambda e: plt.remove(plt.actors[3:]).render())
plt.show(hil, msg, __doc__, viewup='z')




# Copyright (c) 2017, Nicolas P. Rougier https://github.com/rougier/alien-life
# Adapted for vedo by M. Musy, June 2021
#
import numpy as np
from noise._simplex import noise4
from vedo import Plotter, Points, Circle, Text2D, interactive

n = 50000
radius = 200
width, height = (500, 500)
scale = 0.005
t = 0

T = np.random.uniform(0, 2*np.pi, n)
R = np.sqrt(np.random.uniform(0, 1, n))
P = np.zeros((n,2))
X,Y = P[:,0], P[:,1]
X[...] = R*np.cos(T)
Y[...] = R*np.sin(T)
intensity = np.power(1.001-np.sqrt(X**2 + Y**2), 0.75)
X[...] = X*radius + width//2
Y[...] = Y*radius + height//2


def update(evt):
    global t
    t += 0.005
    P_ = np.zeros((n,3))
    cos_t = 1.5*np.cos(2*np.pi * t)
    sin_t = 1.5*np.sin(2*np.pi * t)
    for i in range(n):
        x, y = P[i]
        f = intensity[i]*50
        dx = noise4(scale*x, scale*y, cos_t, sin_t, 2) * f
        dy = noise4(100+scale*x, 200+scale*y, cos_t, sin_t, 2) * f
        P_[i] = [x + dx, y + dy, np.sqrt(dx*dx+dy*dy)/2]
    pts.points(P_)
    plt.render()


pts = Points([X, Y], r=3).alpha(0.8)
cir = Circle(pos=(width/2, height/2, -5), r=radius*1.05)
txt1= Text2D("\Lambda  L  I  E  N    L  I  F  E", s=2.8, pos="top-center")
txt2= Text2D("Original idea by Necessary Disorder", s=0.9, pos="bottom-center")

plt = Plotter()
plt.show(pts, cir, txt1, txt2, elevation=-35, zoom=1.2, interactive=False)
plt.addCallback("timer", update)
plt.timerCallback("create")
interactive()



"""Extract the mesh region that
has the largest connected surface"""
from vedo import *

mesh1 = Volume(dataurl+"embryo.tif").isosurface(80).c("yellow")
printc("area1 =", mesh1.area(), c="yellow")

mesh2 = mesh1.extractLargestRegion().color("lb")
printc("area2 =", mesh2.area(), c="lb")

show(mesh1, __doc__, at=0, shape=(2,1), axes=7)
show(mesh2, at=1, zoom=1.2, interactive=True).close()



from vedo import Volume, dataurl
from vedo.applications import RayCastPlotter

embryo = Volume(dataurl+"embryo.slc").mode(1).c('jet') # change properties

plt = RayCastPlotter(embryo) # Plotter instance
plt.show(viewup="z", bg='black', bg2='blackboard', axes=7).close()



import vtk, sys

__all__ = []
_defs = ""
####################################################################################
####################################################################################
##### To generate documentation you must uncomment this line:
#from vedo.docs_defs import _substitutions_defs as _defs

# Then:
    # cd vedo/docs
    # pip install -r requirements.txt # optionally
    # make html

####################################################################################
####################################################################################


def tips():
    from vedo import colors, __version__
    msg  = " ==========================================================\n"
    msg += "| Press: i     print info about selected object            |\n"
    msg += "|        I     print the RGB color under the mouse         |\n"
    msg += "|        <-->  use arrows to reduce/increase opacity       |\n"
    msg += "|        w/s   toggle wireframe/surface style              |\n"
    msg += "|        p/P   change point size of vertices               |\n"
    msg += "|        l     toggle edges visibility                     |\n"
    msg += "|        x     toggle mesh visibility                      |\n"
    msg += "|        X     invoke a cutter widget tool                 |\n"
    msg += "|        1-2   change mesh color                           |\n"
    msg += "|        3     change mesh texture                         |\n"
    msg += "|        4     use data array as colors, if present        |\n"
    msg += "|        5-6   change background color(s)                  |\n"
    msg += "|        09+-  (on keypad) or +/- to cycle axes style      |\n"
    msg += "|        k     cycle available lighting styles             |\n"
    msg += "|        K     cycle available shading styles              |\n"
    msg += "|        A     toggle anti-aliasing                        |\n"
    msg += "|        D     toggle depth-peeling (for transparencies)   |\n"
    msg += "|        o/O   add/remove light to scene and rotate it     |\n"
    msg += "|        n     show surface mesh normals                   |\n"
    msg += "|        a     toggle interaction to Actor Mode            |\n"
    msg += "|        j     toggle interaction to Joystick Mode         |\n"
    msg += "|        r     reset camera position                       |\n"
    msg += "|        C     print current camera settings               |\n"
    msg += "|        S     save a screenshot                           |\n"
    msg += "|        E     export rendering window to numpy file       |\n"
    msg += "|        q     return control to python script             |\n"
    msg += "|        Esc   abort execution and exit python kernel      |\n"
    msg += "|----------------------------------------------------------|\n"
    msg += "| Mouse: Left-click    rotate scene / pick actors          |\n"
    msg += "|        Middle-click  pan scene                           |\n"
    msg += "|        Right-click   zoom scene in or out                |\n"
    msg += "|        Cntrl-click   rotate scene                        |\n"
    msg += "|----------------------------------------------------------|\n"
    msg += "| Check out documentation at:  https://vedo.embl.es        |\n"
    msg += " =========================================================="
    colors.printc(msg, dim=1)

    msg = " vedo " + __version__ + " "
    colors.printc(msg, invert=1, dim=1, end="")
    vtkVers = vtk.vtkVersion().GetVTKVersion()
    msg = "| vtk " + str(vtkVers)
    msg += " | python " + str(sys.version_info[0]) + "." + str(sys.version_info[1])
    colors.printc(msg, invert=0, dim=1)

####################################################################################
# example web page for X3D
x3d_html = """
<html>
<head>
    <title> My X3DOM web page </title>
    <script type='text/javascript' src='https://www.x3dom.org/download/x3dom.js'> </script>
    <link rel='stylesheet' type='text/css' href='https://www.x3dom.org/download/x3dom.css'/>
</head>
<body><font face="Verdana" size="3">

<h1>My x3d rendering web page</h1>
<p>
    This example loads a 3D scene from file ~fileoutput generated by
    <a href="https://github.com/marcomusy/vedo">vedo</a>
    (see e.g. <a href="https://github.com/marcomusy/vedo/tree/master/examples/other/export_x3d.py">export_x3d.py</a>).

    <br/>
    <br/> - Press t to reset viewpoint.
    <br/> - Press i to reset camera distance.
    <br/> - Right-click and drag mouse to zoom.
    <br/> - Middle-click and drag mouse to pan.
</p>

<x3d width='~widthpx' height='~heightpx'>
    <scene>
        <Inline url="~fileoutput" />
    </scene>
</x3d>

<br clear="all">
<br/>PS: This should work with Firefox/IE.
With Chrome, webgl might not be active by default, then try:<br/><i>
google-chrome --enable-webgl --use-gl=desktop
--log-level=0 --allow-file-access-from-files --allow-file-access ~fileoutput
</i>
</font></body>
</html>
"""


####################################################################################



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

_substitutions_defs = """

.. |gyroscope2.py| replace:: gyroscope2.py
.. _gyroscope2.py: https://github.com/marcomusy/vedo/tree/master/examples/simulations/gyroscope2.py
.. |gyroscope2| image:: https://user-images.githubusercontent.com/32848391/50738942-687b5780-11d9-11e9-97f0-72bbd63f7d6e.gif
    :width: 200 px
    :target: gyroscope2.py_
    :alt: gyroscope2.py

.. |trail.py| replace:: trail.py
.. _trail.py: https://github.com/marcomusy/vedo/tree/master/examples/simulations/trail.py
.. |trail| image:: https://user-images.githubusercontent.com/32848391/58370826-4aee2680-7f0b-11e9-91e6-3120770cfede.gif
    :width: 200 px
    :target: trail.py_
    :alt: trail.py

.. |fillholes.py| replace:: fillholes.py
.. _fillholes.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/fillholes.py

.. |quadratic_morphing.py| replace:: quadratic_morphing.py
.. _quadratic_morphing.py: https://github.com/marcomusy/vedo/tree/master/examples/advanced/quadratic_morphing.py
.. |quadratic_morphing| image:: https://user-images.githubusercontent.com/32848391/50738890-db380300-11d8-11e9-9cef-4c1276cca334.jpg
    :width: 200 px
    :target: quadratic_morphing.py_
    :alt: quadratic_morphing.py

.. |align1.py| replace:: align1.py
.. _align1.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/align1.py
.. |align1| image:: https://user-images.githubusercontent.com/32848391/50738875-c196bb80-11d8-11e9-8bdc-b80fd01a928d.jpg
    :width: 200 px
    :target: align1.py_
    :alt: align1.py

.. |align2.py| replace:: align2.py
.. _align2.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/align2.py
.. |align2| image:: https://user-images.githubusercontent.com/32848391/50738874-c196bb80-11d8-11e9-9587-2177d1680b70.jpg
    :width: 200 px
    :target: align2.py_
    :alt: align2.py

.. |mirror.py| replace:: mirror.py
.. _mirror.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/mirror.py
.. |mirror| image:: https://user-images.githubusercontent.com/32848391/50738855-bf346180-11d8-11e9-97a0-c9aaae6ce052.jpg
    :target: mirror.py_
    :alt: mirror.py

.. |shrink.py| replace:: shrink.py
.. _shrink.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/shrink.py
.. |shrink| image:: https://user-images.githubusercontent.com/32848391/46819143-41042280-cd83-11e8-9492-4f53679887fa.png
    :width: 200 px
    :target: shrink.py_
    :alt: shrink.py

.. |aspring.py| replace:: aspring.py
.. _aspring.py: https://github.com/marcomusy/vedo/tree/master/examples/simulations/aspring.py
.. |aspring| image:: https://user-images.githubusercontent.com/32848391/36788885-e97e80ae-1c8f-11e8-8b8f-ffc43dad1eb1.gif
    :width: 200 px
    :target: aspring.py_
    :alt: aspring.py

.. |delaunay2d.py| replace:: delaunay2d.py
.. _delaunay2d.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/delaunay2d.py
.. |delaunay2d| image:: https://user-images.githubusercontent.com/32848391/50738865-c0658e80-11d8-11e9-8616-b77363aa4695.jpg
    :width: 200 px
    :target: delaunay2d.py_
    :alt: delaunay2d.py

.. |moving_least_squares1D.py| replace:: moving_least_squares1D.py
.. _moving_least_squares1D.py: https://github.com/marcomusy/vedo/tree/master/examples/advanced/moving_least_squares1D.py
.. |moving_least_squares1D| image:: https://user-images.githubusercontent.com/32848391/50738937-61544980-11d9-11e9-8be8-8826032b8baf.jpg
    :width: 200 px
    :target: moving_least_squares1D.py_
    :alt: moving_least_squares1D.py

.. |recosurface.py| replace:: recosurface.py
.. _recosurface.py: https://github.com/marcomusy/vedo/tree/master/examples/advanced/recosurface.py
.. |recosurface| image:: https://user-images.githubusercontent.com/32848391/46817107-b3263880-cd7e-11e8-985d-f5d158992f0c.png
    :target: recosurface.py_
    :alt: recosurface.py

.. |fatlimb.py| replace:: fatlimb.py
.. _fatlimb.py: https://github.com/marcomusy/vedo/tree/master/examples/advanced/fatlimb.py
.. |fatlimb| image:: https://user-images.githubusercontent.com/32848391/50738945-7335ec80-11d9-11e9-9d3f-c6c19df8f10d.jpg
    :width: 200 px
    :target: fatlimb.py_
    :alt: fatlimb.py

.. |largestregion.py| replace:: largestregion.py
.. _largestregion.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/largestregion.py

.. |fitplanes.py| replace:: fitplanes.py
.. _fitplanes.py: https://github.com/marcomusy/vedo/tree/master/examples/advanced/fitplanes.py

.. |mesh_coloring.py| replace:: mesh_coloring.py
.. _mesh_coloring.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/mesh_coloring.py
.. |mesh_coloring| image:: https://user-images.githubusercontent.com/32848391/50738856-bf346180-11d8-11e9-909c-a3f9d32c4e8c.jpg
    :width: 200 px
    :target: mesh_coloring.py_
    :alt: mesh_coloring.py

.. |mesh_alphas.py| replace:: mesh_alphas.py
.. _mesh_alphas.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/mesh_alphas.py
.. |mesh_alphas| image:: https://user-images.githubusercontent.com/32848391/50738857-bf346180-11d8-11e9-80a1-d283aed0b305.jpg
    :width: 200 px
    :target: mesh_alphas.py_
    :alt: mesh_alphas.py

.. |mesh_custom.py| replace:: mesh_custom.py
.. _mesh_custom.py: https://github.com/marcomusy/vedo/tree/master/examples/mesh_custom.py
.. |mesh_custom| image:: https://user-images.githubusercontent.com/32848391/51390972-20d9c180-1b31-11e9-955d-025f1ef24cb7.png
    :width: 200 px
    :target: mesh_custom.py_
    :alt: mesh_custom.py

.. |connVtx.py| replace:: connVtx.py
.. _connVtx.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/connVtx.py
.. |connVtx| image:: https://user-images.githubusercontent.com/32848391/51558919-ec436e00-1e80-11e9-91ac-0787c35fc20e.png
    :width: 200 px
    :target: connVtx.py_
    :alt: connVtx.py

.. |spherical_harmonics1.py| replace:: spherical_harmonics1.py
.. _spherical_harmonics1.py: https://github.com/marcomusy/vedo/tree/master/examples/other/spherical_harmonics1.py

.. |spherical_harmonics2.py| replace:: spherical_harmonics2.py
.. _spherical_harmonics2.py: https://github.com/marcomusy/vedo/tree/master/examples/other/spherical_harmonics2.py

.. |skeletonize.py| replace:: skeletonize.py
.. _skeletonize.py: https://github.com/marcomusy/vedo/tree/master/examples/advanced/skeletonize.py
.. |skeletonize| image:: https://user-images.githubusercontent.com/32848391/46820954-c5f13b00-cd87-11e8-87aa-286528a09de8.png
    :target: spherical_harmonics2.py_
    :alt: skeletonize.py

.. |gyroscope1.py| replace:: gyroscope1.py
.. _gyroscope1.py: https://github.com/marcomusy/vedo/tree/master/examples/simulations/gyroscope1.py
.. |gyroscope1| image:: https://user-images.githubusercontent.com/32848391/39766016-85c1c1d6-52e3-11e8-8575-d167b7ce5217.gif
    :width: 200 px
    :target: gyroscope1.py_
    :alt: gyroscope1.py

.. |icon.py| replace:: icon.py
.. _icon.py: https://github.com/marcomusy/vedo/tree/master/examples/other/icon.py
.. |icon| image:: https://user-images.githubusercontent.com/32848391/50739009-2bfc2b80-11da-11e9-9e2e-a5e0e987a91a.jpg
    :width: 200 px
    :target: icon.py_

.. |lights.py| replace:: lights.py
.. _lights.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/lights.py
    :width: 200 px
    :target: lights.py_
    :alt: lights.py

.. |lorenz.py| replace:: lorenz.py
.. _lorenz.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/lorenz.py
.. |lorenz| image:: https://user-images.githubusercontent.com/32848391/46818115-be7a6380-cd80-11e8-8ffb-60af2631bf71.png
    :width: 200 px
    :target: lorenz.py_
    :alt: lorenz.py

.. |sliders1.py| replace:: sliders1.py
.. _sliders1.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/sliders1.py
.. |sliders1| image:: https://user-images.githubusercontent.com/32848391/50738848-be033480-11d8-11e9-9b1a-c13105423a79.jpg
    :width: 200 px
    :target: sliders1.py_
    :alt: sliders1.py

.. |sliders2.py| replace:: sliders2.py
.. _sliders2.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/sliders2.py

.. |buttons.py| replace:: buttons.py
.. _buttons.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/buttons.py
.. |buttons| image:: https://user-images.githubusercontent.com/32848391/50738870-c0fe2500-11d8-11e9-9b78-92754f5c5968.jpg
    :width: 200 px
    :target: buttons.py_
    :alt: buttons.py

.. |cutter.py| replace:: cutter.py
.. _cutter.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/cutter.py
.. |cutter| image:: https://user-images.githubusercontent.com/32848391/50738866-c0658e80-11d8-11e9-955b-551d4d8b0db5.jpg
    :width: 200 px
    :target: cutter.py_
    :alt: cutter.py

.. |makeVideo.py| replace:: makeVideo.py
.. _makeVideo.py: https://github.com/marcomusy/vedo/tree/master/examples/other/makeVideo.py
.. |makeVideo| image:: https://user-images.githubusercontent.com/32848391/50739007-2bfc2b80-11da-11e9-97e6-620a3541a6fa.jpg
    :width: 200 px
    :target: makeVideo.py_
    :alt: makeVideo.py

.. |fitspheres1.py| replace:: fitspheres1.py
.. _fitspheres1.py: https://github.com/marcomusy/vedo/tree/master/examples/advanced/fitspheres1.py

.. |fitspheres2.py| replace:: fitspheres2.py
.. _fitspheres2.py: https://github.com/marcomusy/vedo/tree/master/examples/advanced/fitspheres2.py
.. |fitspheres2| image:: https://user-images.githubusercontent.com/32848391/50738943-687b5780-11d9-11e9-87a6-054e0fe76241.jpg
    :width: 200 px
    :target: fitspheres2.py_
    :alt: fitspheres2.py

.. |plot_fxy.py| replace:: plot_fxy.py
.. _plot_fxy.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/plot_fxy.py
.. |plot_fxy| image:: https://user-images.githubusercontent.com/32848391/36611824-fd524fac-18d4-11e8-8c76-d3d1b1bb3954.png
    :width: 200 px
    :target: plot_fxy.py_
    :alt: plot_fxy.py

.. |histo_hexagonal.py| replace:: histo_hexagonal.py
.. _histo_hexagonal.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/histo_hexagonal.py
.. |histo_hexagonal| image:: https://user-images.githubusercontent.com/32848391/72434748-b471bc80-379c-11ea-95d7-d70333770582.png
    :width: 200 px
    :target: histo_hexagonal.py_
    :alt: histo_hexagonal.py

.. |histo_1D.py| replace:: histo_1D.py
.. _histo_1D.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/histo_1D.py
.. |histo_1D| image:: https://user-images.githubusercontent.com/32848391/68141260-77cc4e00-ff2d-11e9-9280-0efc5b87314d.png
    :width: 200 px
    :target: histo_1D.py_
    :alt: histo_1D.py

.. |histo_violin.py| replace:: histo_violin.py
.. _histo_violin.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/histo_violin.py
.. |histo_violin| image:: https://user-images.githubusercontent.com/32848391/73481240-b55d3d80-439b-11ea-89a4-6c35ecc84b0d.png
    :width: 200 px
    :target: histo_violin.py_
    :alt: histo_violin.py

.. |align3.py| replace:: align3.py
.. _align3.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/align3.py
.. |align3| image:: https://user-images.githubusercontent.com/32848391/50738873-c196bb80-11d8-11e9-8653-a41108a5f02d.png
    :width: 200 px
    :target: align3.py_
    :alt: align3.py

.. |pca.py| replace:: pca.py
.. _pca.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/pca.py
.. |pca| image:: https://user-images.githubusercontent.com/32848391/50738852-be9bcb00-11d8-11e9-8ac8-ad9278d9cee0.jpg
    :width: 200 px
    :target: pca.py_
    :alt: pca.py

.. |cell_colony.py| replace:: cell_colony.py
.. _cell_colony.py: https://github.com/marcomusy/vedo/tree/master/examples/simulations/cell_colony.py
.. |cell_colony| image:: https://user-images.githubusercontent.com/32848391/50738947-7335ec80-11d9-11e9-9a45-6053b4eaf9f9.jpg
    :width: 200 px
    :target: cell_colony.py_
    :alt: cell_colony.py

.. |mesh_smoother1.py| replace:: mesh_smoother1.py
.. _mesh_smoother1.py: https://github.com/marcomusy/vedo/tree/master/examples/advanced/mesh_smoother1.py
.. |mesh_smoother1| image:: https://user-images.githubusercontent.com/32848391/50738939-67e2c100-11d9-11e9-90cb-716ff3f03f67.jpg
    :width: 200 px
    :target: mesh_smoother1.py_
    :alt: mesh_smoother1.py

.. |moving_least_squares3D.py| replace:: moving_least_squares3D.py
.. _moving_least_squares3D.py: https://github.com/marcomusy/vedo/tree/master/examples/advanced/moving_least_squares3D.py
.. |moving_least_squares3D| image:: https://user-images.githubusercontent.com/32848391/50738935-61544980-11d9-11e9-9c20-f2ce944d2238.jpg
    :width: 200 px
    :target: moving_least_squares3D.py_
    :alt: moving_least_squares3D.py

.. |moving_least_squares2D.py| replace:: moving_least_squares2D.py
.. _moving_least_squares2D.py: https://github.com/marcomusy/vedo/tree/master/examples/advanced/moving_least_squares2D.py
.. |moving_least_squares2D| image:: https://user-images.githubusercontent.com/32848391/50738936-61544980-11d9-11e9-9efb-e2a923762b72.jpg
    :width: 200 px
    :target: moving_least_squares2D.py_
    :alt: moving_least_squares2D.py

.. |boolean.py| replace:: boolean.py
.. _boolean.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/boolean.py
.. |boolean| image:: https://user-images.githubusercontent.com/32848391/50738871-c0fe2500-11d8-11e9-8812-442b69be6db9.png
    :width: 200 px
    :target: boolean.py_
    :alt: boolean.py

.. |surfIntersect.py| replace:: surfIntersect.py
.. _surfIntersect.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/surfIntersect.py

.. |probeLine1.py| replace:: probeLine1.py
.. _probeLine1.py: https://github.com/marcomusy/vedo/tree/master/examples/volumetric/probeLine1.py
.. |probeLine1| image:: https://user-images.githubusercontent.com/32848391/48198460-3aa0a080-e359-11e8-982d-23fadf4de66f.jpg
    :width: 200 px
    :target: probeLine1.py_
    :alt: probeLine1.py

.. |probeLine2.py| replace:: probeLine2.py
.. _probeLine2.py: https://github.com/marcomusy/vedo/tree/master/examples/volumetric/probeLine2.py

.. |slicePlane1.py| replace:: slicePlane1.py
.. _slicePlane1.py: https://github.com/marcomusy/vedo/tree/master/examples/volumetric/slicePlane1.py
.. |slicePlane1| image:: https://user-images.githubusercontent.com/32848391/48198461-3aa0a080-e359-11e8-8c29-18f287f105e6.jpg
    :width: 200 px
    :target: slicePlane1.py_
    :alt: slicePlane1.py

.. |volumeOperations.py| replace:: volumeOperations.py
.. _volumeOperations.py: https://github.com/marcomusy/vedo/tree/master/examples/volumetric/volumeOperations.py
.. |volumeOperations| image:: https://user-images.githubusercontent.com/32848391/48198940-d1ba2800-e35a-11e8-96a7-ffbff797f165.jpg
    :width: 200 px
    :alt: volumeOperations.py

.. |clustering.py| replace:: clustering.py
.. _clustering.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/clustering.py
.. |clustering| image:: https://user-images.githubusercontent.com/32848391/46817286-2039ce00-cd7f-11e8-8b29-42925e03c974.png
    :width: 200 px
    :target: clustering.py_
    :alt: clustering.py

.. |warp1.py| replace:: warp1.py
.. _warp1.py: https://github.com/marcomusy/vedo/tree/master/examples/advanced/warp1.py
.. |warp1| image:: https://user-images.githubusercontent.com/32848391/51403917-34495480-1b52-11e9-956c-918c7805a9b5.png
    :width: 200 px
    :target: warp1.py_
    :alt: warp1.py

.. |colorcubes.py| replace:: colorcubes.py
.. _colorcubes.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/colorcubes.py
.. |colorcubes| image:: https://user-images.githubusercontent.com/32848391/50738867-c0658e80-11d8-11e9-9e05-ac69b546b7ec.png
    :width: 200 px
    :target: colorcubes.py_
    :alt: colorcubes.py

.. |colorpalette.py| replace:: colorpalette.py
.. _colorpalette.py: https://github.com/marcomusy/vedo/tree/master/examples/other/colorpalette.py
.. |colorpalette| image:: https://user-images.githubusercontent.com/32848391/50739011-2c94c200-11da-11e9-8f36-ede1b2a014a8.jpg
    :width: 200 px
    :target: colorpalette.py
    :alt: colorpalette.py

.. |colormaps| image:: https://user-images.githubusercontent.com/32848391/50738804-577e1680-11d8-11e9-929e-fca17a8ac6f3.jpg
    :width: 450 px
    :alt: colormaps

.. |colorprint.py| replace:: printc.py
.. _colorprint.py: https://github.com/marcomusy/vedo/tree/master/examples/other/printc.py
.. |colorprint| image:: https://user-images.githubusercontent.com/32848391/50739010-2bfc2b80-11da-11e9-94de-011e50a86e61.jpg
    :target: colorprint.py_
    :alt: colorprint.py

.. |ribbon.py| replace:: ribbon.py
.. _ribbon.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/ribbon.py
.. |ribbon| image:: https://user-images.githubusercontent.com/32848391/50738851-be9bcb00-11d8-11e9-80ee-bd73c1c29c06.jpg
    :width: 200 px
    :target: ribbon.py_
    :alt: ribbon.py

.. |manyspheres.py| replace:: manyspheres.py
.. _manyspheres.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/manyspheres.py
.. |manyspheres| image:: https://user-images.githubusercontent.com/32848391/46818673-1f566b80-cd82-11e8-9a61-be6a56160f1c.png
    :target: manyspheres.py_

.. |manypoints.py| replace:: manypoints.py
.. _manypoints.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/manypoints.py

.. |earth.py| replace:: earth.py
.. _earth.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/earth.py
.. |earth| image:: https://user-images.githubusercontent.com/32848391/51031592-5a448700-159d-11e9-9b66-bee6abb18679.png
    :width: 200 px
    :target: earth.py_
    :alt: earth.py

.. |brownian2D.py| replace:: brownian2D.py
.. _brownian2D.py: https://github.com/marcomusy/vedo/tree/master/examples/simulations/brownian2D.py
.. |brownian2D| image:: https://user-images.githubusercontent.com/32848391/50738948-73ce8300-11d9-11e9-8ef6-fc4f64c4a9ce.gif
    :width: 200 px
    :target: brownian2D.py_

.. |gas.py| replace:: gas.py
.. _gas.py: https://github.com/marcomusy/vedo/tree/master/examples/simulations/gas.py
.. |gas| image:: https://user-images.githubusercontent.com/32848391/50738954-7e891800-11d9-11e9-95aa-67c92ca6476b.gif
    :width: 200 px
    :target: gas.py_
    :alt: gas.py

.. |tube.py| replace:: tube.py
.. _tube.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/tube.py
.. |tube| image:: https://user-images.githubusercontent.com/32848391/51801626-adc30000-2240-11e9-8866-9d9d5d8790ab.png
    :width: 200 px
    :target: tube.py_
    :alt: tube.py

.. |mesh_threshold.py| replace:: mesh_threshold.py
.. _mesh_threshold.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/mesh_threshold.py
.. |mesh_threshold| image:: https://user-images.githubusercontent.com/32848391/51807663-4762cf80-228a-11e9-9d0c-184bb11a97bf.png
    :width: 200 px
    :target: mesh_threshold.py_
    :alt: mesh_threshold.py

.. |cutWithMesh1.py| replace:: cutWithMesh1.py
.. _cutWithMesh1.py: https://github.com/marcomusy/vedo/tree/master/examples/advanced/cutWithMesh1.py
.. |cutWithMesh1| image:: https://user-images.githubusercontent.com/32848391/51808907-e7c0f000-229a-11e9-98a7-fefc7261b3c3.png
    :width: 200 px
    :target: cutWithMesh1.py_
    :alt: cutWithMesh1.py

.. |paraboloid| image:: https://user-images.githubusercontent.com/32848391/51211547-260ef480-1916-11e9-95f6-4a677e37e355.png
    :width: 200 px
    :alt: paraboloid

.. |isosurfaces.py| replace:: isosurfaces.py
.. _isosurfaces.py: https://github.com/marcomusy/vedo/tree/master/examples/volumetric/isosurfaces.py
.. |isosurfaces| image:: https://user-images.githubusercontent.com/32848391/51558920-ec436e00-1e80-11e9-9d96-aa9b7c72d58b.png
    :width: 200 px
    :target: isosurfaces.py_
    :alt: isosurfaces.py

.. |meshquality.py| replace:: meshquality.py
.. _meshquality.py: https://github.com/marcomusy/vedo/tree/master/examples/advanced/meshquality.py
.. |meshquality| image:: https://user-images.githubusercontent.com/32848391/51831269-fb4b7580-22f1-11e9-81ea-13467a5649ca.png
    :width: 200 px
    :target: meshquality.py_
    :alt: meshquality.py

.. |geodesic.py| replace:: geodesic.py
.. _geodesic.py: https://github.com/marcomusy/vedo/tree/master/examples/advanced/geodesic.py
.. |geodesic| image:: https://user-images.githubusercontent.com/32848391/51855637-015f4780-232e-11e9-92ca-053a558e7f70.png
    :width: 200 px
    :target: geodesic.py_
    :alt: geodesic.py


.. |cutAndCap.py| replace:: cutAndCap.py
.. _cutAndCap.py: https://github.com/marcomusy/vedo/tree/master/examples/advanced/cutAndCap.py
.. |cutAndCap| image:: https://user-images.githubusercontent.com/32848391/51930515-16ee7300-23fb-11e9-91af-2b6b3d626246.png
    :width: 200 px
    :target: cutAndCap.py_
    :alt: cutAndCap.py


.. |convexHull.py| replace:: convexHull.py
.. _convexHull.py: https://github.com/marcomusy/vedo/tree/master/examples/advanced/convexHull.py
.. |convexHull| image:: https://user-images.githubusercontent.com/32848391/51932732-068cc700-2400-11e9-9b68-30294a4fa4e3.png
    :width: 200 px
    :target: convexHull.py_
    :alt: convexHull.py

.. |curvature| image:: https://user-images.githubusercontent.com/32848391/51934810-c2e88c00-2404-11e9-8e7e-ca0b7984bbb7.png
    :alt: curvature

.. |progbar| image:: https://user-images.githubusercontent.com/32848391/51858823-ed1f4880-2335-11e9-8788-2d102ace2578.png
    :alt: progressbar

.. |multiwindows| image:: https://user-images.githubusercontent.com/32848391/50738853-be9bcb00-11d8-11e9-9c8e-69864ad7c045.jpg
    :alt: multiwindows

.. |annotations.py| replace:: annotations.py
.. _annotations.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/annotations.py

.. |Cone| image:: https://raw.githubusercontent.com/lorensen/VTKExamples/master/src/Testing/Baseline/Cxx/GeometricObjects/TestCone.png
    :width: 200 px

.. |Cylinder| image:: https://raw.githubusercontent.com/lorensen/VTKExamples/master/src/Testing/Baseline/Cxx/GeometricObjects/TestCylinder.png
    :width: 200 px

.. |Disk| image:: https://raw.githubusercontent.com/lorensen/VTKExamples/master/src/Testing/Baseline/Cxx/GeometricObjects/TestDisk.png
    :width: 200 px

.. |OrientedArrow| image:: https://raw.githubusercontent.com/lorensen/VTKExamples/master/src/Testing/Baseline/Cxx/GeometricObjects/TestOrientedArrow.png
    :width: 200 px

.. |Plane| image:: https://raw.githubusercontent.com/lorensen/VTKExamples/master/src/Testing/Baseline/Cxx/GeometricObjects/TestPlane.png
    :width: 200 px

.. |Polygon| image:: https://raw.githubusercontent.com/lorensen/VTKExamples/master/src/Testing/Baseline/Cxx/GeometricObjects/TestRegularPolygonSource.png
    :width: 200 px

.. |Sphere| image:: https://raw.githubusercontent.com/lorensen/VTKExamples/master/src/Testing/Baseline/Cxx/GeometricObjects/TestSphere.png
    :width: 200 px


.. |embryoslider| image:: https://user-images.githubusercontent.com/32848391/52141624-975ce000-2656-11e9-8d31-2a3c92ab79d6.png
    :width: 200 px

.. |isosurfaces1| image:: https://user-images.githubusercontent.com/32848391/52141625-975ce000-2656-11e9-91fc-291e072fc4c1.png
    :width: 200 px

.. |splitmesh.py| replace:: splitmesh.py
.. _splitmesh.py: https://github.com/marcomusy/vedo/tree/master/examples/advanced/splitmesh.py
.. |splitmesh| image:: https://user-images.githubusercontent.com/32848391/52141626-97f57680-2656-11e9-80ea-fcd3571a6422.png
    :width: 200 px
    :target: splitmesh.py_
    :alt: splitmesh.py

.. |projectsphere.py| replace:: projectsphere.py
.. _projectsphere.py: https://github.com/marcomusy/vedo/tree/master/examples/advanced/projectsphere.py
.. |projectsphere| image:: https://user-images.githubusercontent.com/32848391/52144163-c9be0b80-265d-11e9-9ce6-d6f2b919c214.png
    :width: 200 px
    :target: projectsphere.py_
    :alt: projectsphere.py


.. |mesh2volume.py| replace:: mesh2volume.py
.. _mesh2volume.py: https://github.com/marcomusy/vedo/tree/master/examples/volumetric/mesh2volume.py
.. |mesh2volume| image:: https://user-images.githubusercontent.com/32848391/52168902-5638fe80-2730-11e9-8033-8e470a3d4f0f.jpg
    :width: 200 px
    :target: mesh2volume.py_
    :alt: mesh2volume.py

.. |markpoint.py| replace:: markpoint.py
.. _markpoint.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/markpoint.py
.. |markpoint| image:: https://user-images.githubusercontent.com/32848391/52169969-1fb7af80-2741-11e9-937f-5c331d9a1d11.jpg
    :width: 200 px
    :target: markpoint.py_
    :alt: markpoint.py

.. |readVolumeAsIsoSurface.py| replace:: readVolumeAsIsoSurface.py
.. _readVolumeAsIsoSurface.py: https://github.com/marcomusy/vedo/tree/master/examples/volumetric/readVolumeAsIsoSurface.py

.. |read_volume2.py| replace:: read_volume2.py
.. _read_volume2.py: https://github.com/marcomusy/vedo/tree/master/examples/volumetric/read_volume2.py
.. |read_volume2| image:: https://user-images.githubusercontent.com/32848391/50739036-6bc31300-11da-11e9-89b3-04a75187f812.jpg
    :width: 200 px
    :target: read_volume2.py_
    :alt: read_volume2.py

.. |glyphs.py| replace:: glyphs.py
.. _glyphs.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/glyphs.py
.. |glyphs| image:: https://user-images.githubusercontent.com/32848391/52233403-47cd1d00-28bf-11e9-86b0-cbceebbde0de.jpg
    :width: 200 px
    :target: glyphs.py_
    :alt: glyphs.py

.. |glyphs_arrows.py| replace:: glyphs_arrows.py
.. _glyphs_arrows.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/glyphs_arrows.py
.. |glyphs_arrows| image:: https://user-images.githubusercontent.com/32848391/55897850-a1a0da80-5bc1-11e9-81e0-004c8f396b43.jpg
    :width: 200 px
    :target: glyphs_arrows.py_
    :alt: glyphs_arrows.py

.. |interpolateField.py| replace:: interpolateField.py
.. _interpolateField.py: https://github.com/marcomusy/vedo/tree/master/examples/advanced/interpolateField.py
.. |interpolateField| image:: https://user-images.githubusercontent.com/32848391/52416117-25b6e300-2ae9-11e9-8d86-575b97e543c0.png
    :width: 200 px
    :target: interpolateField.py_
    :alt: interpolateField.py

.. |rotateImage.py| replace:: rotateImage.py
.. _rotateImage.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/rotateImage.py
.. |rotateImage| image:: https://user-images.githubusercontent.com/32848391/52416910-fb662500-2aea-11e9-88ba-9c73128e8666.jpg
    :width: 200 px
    :target: rotateImage.py_
    :alt: rotateImage.py

.. |basicshapes| image:: https://user-images.githubusercontent.com/32848391/50738811-58af4380-11d8-11e9-9bfb-378c27c9d26f.png
    :alt: basicshapes

.. |lines| image:: https://user-images.githubusercontent.com/32848391/52503049-ac9cb600-2be4-11e9-86af-72a538af14ef.png
    :width: 200 px
    :alt: lines

.. |vlogo_large| image:: https://user-images.githubusercontent.com/32848391/52522716-4fa70b80-2c89-11e9-92a7-0d22cbe34758.png
    :alt: vlogo_large

.. |vlogo_medium| image:: https://user-images.githubusercontent.com/32848391/52522717-503fa200-2c89-11e9-87ab-67eb44652e24.png
    :alt: vlogo_medium

.. |vlogo_small| image:: https://user-images.githubusercontent.com/32848391/52522718-50d83880-2c89-11e9-80ff-df1b5618a84a.png
    :alt: vlogo_small

.. |vlogo_small_dark| image:: https://user-images.githubusercontent.com/32848391/52522719-50d83880-2c89-11e9-8b90-a1c21c27b007.png
    :alt: vlogo_small_dark

.. |vlogo_tube| image:: https://user-images.githubusercontent.com/32848391/52522720-5170cf00-2c89-11e9-8b1d-a7a5cf75e71b.png
    :alt: vlogo_tube

.. |vlogo_tube_dark| image:: https://user-images.githubusercontent.com/32848391/52522721-5170cf00-2c89-11e9-8fbb-6efa13940aa1.png
    :alt: vlogo_tube_dark

.. |fitline.py| replace:: fitline.py
.. _fitline.py: https://github.com/marcomusy/vedo/tree/master/examples/advanced/fitline.py
.. |fitline| image:: https://user-images.githubusercontent.com/32848391/50738864-c0658e80-11d8-11e9-8754-c670f1f331d6.jpg
    :width: 200 px
    :target: fitline.py_
    :alt: fitline.py

.. |sliders3d.py| replace:: sliders3d.py
.. _sliders3d.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/sliders3d.py
.. |sliders3d| image:: https://user-images.githubusercontent.com/32848391/52859555-4efcf200-312d-11e9-9290-6988c8295163.png
    :width: 200 px
    :target: sliders3d.py_
    :alt: sliders3d.py

.. |ex01_showmesh.py| replace:: ex01_showmesh.py
.. _ex01_showmesh.py: https://github.com/marcomusy/vedo/tree/master/examples/other/dolfin/ex01_show-mesh.py
.. |ex01_showmesh| image:: https://user-images.githubusercontent.com/32848391/53026243-d2d31900-3462-11e9-9dde-518218c241b6.jpg
    :width: 200 px
    :target: ex01_showmesh.py_
    :alt: ex01_showmesh.py

.. |ex02_tetralize-mesh.py| replace:: ex02_tetralize-mesh.py
.. _ex02_tetralize-mesh.py: https://github.com/marcomusy/vedo/tree/master/examples/other/dolfin/ex02_tetralize-mesh.py
.. |ex02_tetralize-mesh| image:: https://user-images.githubusercontent.com/32848391/53026244-d2d31900-3462-11e9-835a-1fa9d66d3dae.png
    :width: 200 px
    :target: ex02_tetralize-mesh.py_
    :alt: ex02_tetralize-mesh.py

.. |ex06_elasticity1.py| replace:: ex06_elasticity1.py
.. _ex06_elasticity1.py: https://github.com/marcomusy/vedo/tree/master/examples/other/dolfin/ex06_elasticity1.py
.. |ex06_elasticity1| image:: https://user-images.githubusercontent.com/32848391/53026245-d2d31900-3462-11e9-9db4-96211569d114.jpg
    :width: 200 px
    :target: ex06_elasticity1.py_
    :alt: ex06_elasticity1.py

.. |ex06_elasticity2.py| replace:: ex06_elasticity2.py
.. _ex06_elasticity2.py: https://github.com/marcomusy/vedo/tree/master/examples/other/dolfin/ex06_elasticity2.py
.. |ex06_elasticity2| image:: https://user-images.githubusercontent.com/32848391/53026246-d36baf80-3462-11e9-96a5-8eaf0bb0f9a4.jpg
    :width: 200 px
    :target: ex06_elasticity2.py_
    :alt: ex06_elasticity2.py


.. |flatarrow.py| replace:: flatarrow.py
.. _flatarrow.py: https://github.com/marcomusy/vedo/tree/master/examples/other/basic/flatarrow.py
.. |flatarrow| image:: https://user-images.githubusercontent.com/32848391/54612632-97c00780-4a59-11e9-8532-940c25a5dfd8.png
    :width: 200 px
    :target: flatarrow.py_
    :alt: flatarrow.py

.. |printhisto| image:: https://user-images.githubusercontent.com/32848391/55073046-03732780-508d-11e9-9bf9-c5de8631dd73.png
    :width: 200 px

.. |distance2mesh.py| replace:: distance2mesh.py
.. _distance2mesh.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/distance2mesh.py
.. |distance2mesh| image:: https://user-images.githubusercontent.com/32848391/55965881-b5a71380-5c77-11e9-8680-5bddceab813a.png
    :width: 200 px
    :target: distance2mesh.py_
    :alt: distance2mesh.py

.. |pendulum.py| replace:: pendulum.py
.. _pendulum.py: https://github.com/marcomusy/vedo/tree/master/examples/simulations/pendulum.py
.. |pendulum| image:: https://user-images.githubusercontent.com/32848391/55420020-51e56200-5576-11e9-8513-4a5d93913b17.png
    :width: 200 px
    :target: pendulum.py_
    :alt: pendulum.py

.. |latex.py| replace:: latex.py
.. _latex.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/latex.py
.. |latex| image:: https://user-images.githubusercontent.com/32848391/55568648-6190b200-5700-11e9-9547-0798c588a7a5.png
    :width: 200 px
    :target: latex.py_
    :alt: latex.py

.. |ft04_heat_gaussian.py| replace:: ft04_heat_gaussian.py
.. _ft04_heat_gaussian.py: https://github.com/marcomusy/vedo/tree/master/examples/other/dolfin/ft04_heat_gaussian.py
.. |ft04_heat_gaussian| image:: https://user-images.githubusercontent.com/32848391/55578167-88a5ae80-5715-11e9-84ea-bdab54099887.gif
    :width: 200 px
    :target: ft04_heat_gaussian.py_
    :alt: ft04_heat_gaussian.py

.. |cutcube| image:: https://user-images.githubusercontent.com/32848391/55965516-08cc9680-5c77-11e9-8d23-720f6c088ea2.png
    :width: 200 px

.. |intline| image:: https://user-images.githubusercontent.com/32848391/55967065-eee08300-5c79-11e9-8933-265e1bab9f7e.png
    :width: 200 px

.. |cropped| image:: https://user-images.githubusercontent.com/32848391/57081955-0ef1e800-6cf6-11e9-99de-b45220939bc9.png
    :width: 200 px

.. |dolfinmesh| image:: https://user-images.githubusercontent.com/32848391/53026243-d2d31900-3462-11e9-9dde-518218c241b6.jpg
    :width: 200 px

.. |turing_pattern.py| replace:: turing_pattern.py
.. _turing_pattern.py: https://github.com/marcomusy/vedo/tree/master/examples/other/dolfin/turing_pattern.py
.. |turing_pattern| image:: https://user-images.githubusercontent.com/32848391/56056437-77cfeb00-5d5c-11e9-9887-828e5745d547.gif
    :width: 200 px
    :target: turing_pattern.py_
    :alt: turing_pattern.py

.. |demo_cahn-hilliard.py| replace:: demo_cahn-hilliard.py
.. _demo_cahn-hilliard.py: https://github.com/marcomusy/vedo/tree/master/examples/other/dolfin/demo_cahn-hilliard.py
.. |demo_cahn-hilliard| image:: https://user-images.githubusercontent.com/32848391/56664730-edb34b00-66a8-11e9-9bf3-73431f2a98ac.gif
    :width: 200 px
    :target: demo_cahn-hilliard.py_
    :alt: demo_cahn-hilliard.py


.. |navier-stokes_lshape.py| replace:: navier-stokes_lshape.py
.. _navier-stokes_lshape.py: https://github.com/marcomusy/vedo/tree/master/examples/other/dolfin/navier-stokes_lshape.py
.. |navier-stokes_lshape| image:: https://user-images.githubusercontent.com/32848391/56671156-6bc91f00-66b4-11e9-8c58-e6b71e2ad1d0.gif
    :width: 200 px
    :target: navier-stokes_lshape.py_
    :alt: navier-stokes_lshape.py


.. |mesh_map2cell.py| replace:: mesh_map2cell.py
.. _mesh_map2cell.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/mesh_map2cell.py
.. |mesh_map2cell| image:: https://user-images.githubusercontent.com/32848391/56600859-0153a880-65fa-11e9-88be-34fd96b18e9a.png
    :width: 200 px
    :target: mesh_map2cell.py_
    :alt: mesh_map2cell.py


.. |ex03_poisson.py| replace:: ex03_poisson.py
.. _ex03_poisson.py: https://github.com/marcomusy/vedo/tree/master/examples/other/dolfin/ex03_poisson.py
.. |ex03_poisson| image:: https://user-images.githubusercontent.com/32848391/54925524-bec18200-4f0e-11e9-9eab-29fd61ef3b8e.png
    :width: 200 px
    :target: ex03_poisson.py_
    :alt: ex03_poisson.py

.. |elastodynamics.py| replace:: elastodynamics.py
.. _elastodynamics.py: https://github.com/marcomusy/vedo/tree/master/examples/other/dolfin/elastodynamics.py
.. |elastodynamics| image:: https://user-images.githubusercontent.com/32848391/54932788-bd4a8680-4f1b-11e9-9326-33645171a45e.gif
    :width: 200 px
    :target: elastodynamics.py_
    :alt: elastodynamics.py

.. |ft02_poisson_membrane.py| replace:: ft02_poisson_membrane.py
.. _ft02_poisson_membrane.py: https://github.com/marcomusy/vedo/tree/master/examples/other/dolfin/ft02_poisson_membrane.py
.. |ft02_poisson_membrane| image:: https://user-images.githubusercontent.com/32848391/55499287-ed91d380-5645-11e9-8e9a-e31e2e3b1649.jpg
    :width: 200 px
    :target: ft02_poisson_membrane.py_
    :alt: ft02_poisson_membrane.py


.. |stokes.py| replace:: stokes.py
.. _stokes.py: https://github.com/marcomusy/vedo/tree/master/examples/other/dolfin/stokes.py
.. |stokes| image:: https://user-images.githubusercontent.com/32848391/55098209-aba0e480-50bd-11e9-8842-42d3f0b2d9c8.png
    :width: 200 px
    :target: stokes.py_
    :alt: stokes.py

.. |stokes1.py| replace:: stokes1.py
.. _stokes1.py: https://github.com/marcomusy/vedo/tree/master/examples/other/dolfin/stokes.py
.. |stokes1| image:: https://user-images.githubusercontent.com/32848391/56986911-6116fa00-6b8c-11e9-83f5-5b4efe430c0c.jpg
    :width: 200 px
    :target: stokes1.py_
    :alt: stokes1.py

.. |demo_submesh.py| replace:: demo_submesh.py
.. _demo_submesh.py: https://github.com/marcomusy/vedo/tree/master/examples/other/dolfin/demo_submesh.py
.. |demo_submesh| image:: https://user-images.githubusercontent.com/32848391/56675428-4e984e80-66bc-11e9-90b0-43dde7e4cc29.png
    :width: 200 px
    :target: demo_submesh.py_
    :alt: demo_submesh.py

.. |pi_estimate.py| replace:: pi_estimate.py
.. _pi_estimate.py: https://github.com/marcomusy/vedo/tree/master/examples/other/dolfin/pi_estimate.py
.. |pi_estimate| image:: https://user-images.githubusercontent.com/32848391/56675429-4e984e80-66bc-11e9-9217-a0652a8e74fe.png
    :width: 200 px
    :target: pi_estimate.py_
    :alt: pi_estimate.py

.. |isolines.py| replace:: isolines.py
.. _isolines.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/isolines.py
.. |isolines| image:: https://user-images.githubusercontent.com/32848391/72433087-f00a8780-3798-11ea-9778-991f0abeca70.png
    :width: 200 px
    :target: isolines.py_
    :alt: isolines.py

.. |inset.py| replace:: inset.py
.. _inset.py: https://github.com/marcomusy/vedo/tree/master/examples/other/inset.py
.. |inset| image:: https://user-images.githubusercontent.com/32848391/56758560-3c3f1300-6797-11e9-9b33-49f5a4876039.jpg
    :width: 200 px
    :target: inset.py_
    :alt: inset.py

.. |legosurface.py| replace:: legosurface.py
.. _legosurface.py: https://github.com/marcomusy/vedo/tree/master/examples/volumetric/legosurface.py
.. |legosurface| image:: https://user-images.githubusercontent.com/32848391/56820682-da40e500-684c-11e9-8ea3-91cbcba24b3a.png
    :width: 200 px
    :target: legosurface.py_
    :alt: legosurface.py


.. |streamribbons.py| replace:: streamribbons.py
.. _streamribbons.py: https://github.com/marcomusy/vedo/tree/master/examples/volumetric/streamribbons.py
.. |streamribbons| image:: https://user-images.githubusercontent.com/32848391/56963999-9145a500-6b5a-11e9-9461-0037c471faab.png
    :width: 200 px
    :target: streamribbons.py_
    :alt: streamribbons.py


.. |streamlines1.py| replace:: streamlines1.py
.. _streamlines1.py: https://github.com/marcomusy/vedo/tree/master/examples/volumetric/streamlines1.py
.. |streamlines1| image:: https://user-images.githubusercontent.com/32848391/56964002-9145a500-6b5a-11e9-9e3f-da712609d896.png
    :width: 200 px
    :target: streamlines1.py_
    :alt: streamlines1.py

.. |streamlines2.py| replace:: streamlines2.py
.. _streamlines2.py: https://github.com/marcomusy/vedo/tree/master/examples/volumetric/streamlines2.py
.. |streamlines2| image:: https://user-images.githubusercontent.com/32848391/56964001-9145a500-6b5a-11e9-935b-1b2425bd7dd2.png
    :width: 200 px
    :target: streamlines2.py_
    :alt: streamlines2.py

.. |office.py| replace:: office.py
.. _office.py: https://github.com/marcomusy/vedo/tree/master/examples/volumetric/office.py
.. |office| image:: https://user-images.githubusercontent.com/32848391/56964003-9145a500-6b5a-11e9-9d9e-9736d90e1900.png
    :width: 200 px
    :target: office.py_
    :alt: office.py

.. |value-iteration.py| replace:: value-iteration.py
.. _value-iteration.py: https://github.com/marcomusy/vedo/tree/master/examples/other/value-iteration.py
.. |value-iteration| image:: https://user-images.githubusercontent.com/32848391/56964055-afaba080-6b5a-11e9-99cf-3fac99df9878.jpg
    :width: 200 px
    :target: value-iteration.py_
    :alt: value-iteration.py

.. |magnetostatics.py| replace:: magnetostatics.py
.. _magnetostatics.py: https://github.com/marcomusy/vedo/tree/master/examples/other/dolfin/magnetostatics.py
.. |magnetostatics| image:: https://user-images.githubusercontent.com/32848391/56985162-fb287380-6b87-11e9-9cf9-045bd08c3b9b.jpg
    :width: 200 px
    :target: magnetostatics.py_
    :alt: magnetostatics.py

.. |export_x3d.py| replace:: export_x3d.py
.. _export_x3d.py: https://github.com/marcomusy/vedo/tree/master/examples/other/export_x3d.py
.. |export_x3d| image:: https://user-images.githubusercontent.com/32848391/57160341-c6ffbd80-6de8-11e9-95ff-7215ce642bc5.jpg
    :width: 200 px
    :target: export_x3d.py_
    :alt: export_x3d.py


.. |silhouette.py| replace:: silhouette.py
.. _silhouette.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/silhouette1.py
.. |silhouette| image:: https://user-images.githubusercontent.com/32848391/57179369-8e5df380-6e7d-11e9-99b4-3b1a120dd375.png
    :width: 200 px
    :target: silhouette.py_
    :alt: silhouette.py

.. |shadow.py| replace:: shadow.py
.. _shadow.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/shadow.py
.. |shadow| image:: https://user-images.githubusercontent.com/32848391/57312574-1d714280-70ee-11e9-8741-04fc5386d692.png
    :width: 200 px
    :target: shadow.py_
    :alt: shadow.py

.. |airplanes.py| replace:: airplanes.py
.. _airplanes.py: https://github.com/marcomusy/vedo/tree/master/examples/simulations/airplanes.py
.. |airplanes| image:: https://user-images.githubusercontent.com/32848391/57341963-b8910900-713c-11e9-898a-84b6d3712bce.gif
    :width: 200 px
    :target: airplanes.py_
    :alt: airplanes.py


.. |heatconv.py| replace:: heatconv.py
.. _heatconv.py: https://github.com/marcomusy/vedo/tree/master/examples/other/dolfin/heatconv.py
.. |heatconv| image:: https://user-images.githubusercontent.com/32848391/57455107-b200af80-726a-11e9-897d-9c7bcb9854ac.gif
    :width: 200 px
    :target: heatconv.py_
    :alt: heatconv.py

.. |scalemesh.py| replace:: scalemesh.py
.. _scalemesh.py: https://github.com/marcomusy/vedo/tree/master/examples/other/dolfin/scalemesh.py
.. |scalemesh| image:: https://user-images.githubusercontent.com/32848391/57393382-431c4b80-71c3-11e9-9a2c-8abb172f5468.png
    :width: 200 px
    :target: scalemesh.py_
    :alt: scalemesh.py

.. |elasticbeam.py| replace:: elasticbeam.py
.. _elasticbeam.py: https://github.com/marcomusy/vedo/tree/master/examples/other/dolfin/elasticbeam.py
.. |elasticbeam| image:: https://user-images.githubusercontent.com/32848391/57476429-d7a3ae00-7296-11e9-9f50-8f456823ef3d.png
    :width: 200 px
    :target: elasticbeam.py_
    :alt: elasticbeam.py

.. |specular.py| replace:: specular.py
.. _specular.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/specular.py
.. |specular| image:: https://user-images.githubusercontent.com/32848391/57543051-8c030a00-7353-11e9-84cd-b01f3449d255.jpg
    :width: 200 px
    :target: specular.py_
    :alt: specular.py

.. |wavy_1d.py| replace:: wavy_1d.py
.. _wavy_1d.py: https://github.com/marcomusy/vedo/tree/master/examples/other/dolfin/wavy_1d.py
.. |wavy_1d| image:: https://user-images.githubusercontent.com/32848391/57570898-70593b80-7407-11e9-87cf-ce498f499c09.gif
    :width: 200 px
    :target: wavy_1d.py_
    :alt: wavy_1d.py

.. |idealpass.link| replace:: idealpass.link
.. _idealpass.link: https://lorensen.github.io/VTKExamples/site/Cxx/ImageProcessing/IdealHighPass
.. |idealpass| image:: https://raw.githubusercontent.com/lorensen/VTKExamples/master/src/Testing/Baseline/Cxx/ImageProcessing/TestIdealHighPass.png
    :width: 200 px
    :target: idealpass.link_

.. |buildmesh.py| replace:: buildmesh.py
.. _buildmesh.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/buildmesh.py
.. |buildmesh| image:: https://user-images.githubusercontent.com/32848391/57858625-b0e2fb80-77f1-11e9-94f0-1973ed86ae70.png
    :width: 200 px
    :target: buildmesh.py_
    :alt: buildmesh.py

.. |customAxes1.py| replace:: customAxes1.py
.. _customAxes1.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/customAxes1.py
.. |customAxes1| image:: https://user-images.githubusercontent.com/32848391/58181826-c605d180-7cac-11e9-9786-11b5eb278f20.png
    :width: 200 px
    :target: customAxes1.py_
    :alt: customAxes1.py

.. |customAxes2.py| replace:: customAxes2.py
.. _customAxes2.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/customAxes2.py

.. |customAxes3.py| replace:: customAxes3.py
.. _customAxes3.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/customAxes3.py

.. |awefem.py| replace:: awefem.py
.. _awefem.py: https://github.com/marcomusy/vedo/tree/master/examples/other/dolfin/awefem.py
.. |awefem| image:: https://user-images.githubusercontent.com/32848391/58368591-8b3fab80-7eef-11e9-882f-8b8eaef43567.gif
    :width: 200 px
    :target: awefem.py_
    :alt: awefem.py

.. |fenics_logo| image:: https://user-images.githubusercontent.com/32848391/58764910-3940fa80-856d-11e9-8160-af89a5ab5d02.gif

.. |warp3.py| replace:: warp3.py
.. _warp3.py: https://github.com/marcomusy/vedo/tree/master/examples/advanced/warp3.py
.. |warp3| image:: https://user-images.githubusercontent.com/32848391/59032715-385ae200-8867-11e9-9b07-7f4f8fbfa5bd.png
    :width: 200 px
    :target: warp3.py_
    :alt: warp3.py

.. |interpolateVolume.py| replace:: interpolateVolume.py
.. _interpolateVolume.py: https://github.com/marcomusy/vedo/tree/master/examples/volumetric/interpolateVolume.py
.. |interpolateVolume| image:: https://user-images.githubusercontent.com/32848391/59095175-1ec5a300-8918-11e9-8bc0-fd35c8981e2b.jpg
    :width: 200 px
    :target: interpolateVolume.py_
    :alt: interpolateVolume.py

.. |deleteMeshPoints.py| replace:: deleteMeshPoints.py
.. _deleteMeshPoints.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/deleteMeshPoints.py
.. |deleteMeshPoints| image:: https://user-images.githubusercontent.com/32848391/59768041-e9b83980-92a3-11e9-94f4-ca1f92540e9f.png
    :width: 200 px
    :target: deleteMeshPoints.py_
    :alt: deleteMeshPoints.py

.. |gray_scott.ipynb| replace:: gray_scott.ipynb
.. _gray_scott.ipynb: https://github.com/marcomusy/vedo/tree/master/examples/simulations/gray_scott.ipynb
.. |gray_scott| image:: https://user-images.githubusercontent.com/32848391/59788744-aaeaa980-92cc-11e9-825d-58da26ca21ff.gif
    :width: 200 px
    :target: gray_scott.ipynb_
    :alt: gray_scott.ipynb

.. |volterra.py| replace:: volterra.py
.. _volterra.py: https://github.com/marcomusy/vedo/tree/master/examples/simulations/volterra.py
.. |volterra| image:: https://user-images.githubusercontent.com/32848391/59788745-aaeaa980-92cc-11e9-93d5-f6a577ba5e4d.png
    :width: 200 px
    :target: volterra.py_
    :alt: volterra.py

.. |tensors.py| replace:: tensors.py
.. _tensors.py: https://github.com/marcomusy/vedo/tree/master/examples/volumetric/tensors.py
.. |tensors| image:: https://user-images.githubusercontent.com/32848391/59944747-e2d92480-9465-11e9-8012-1fc34a2e30c6.png
    :width: 200 px
    :target: tensors.py_
    :alt: tensors.py

.. |tensor_grid.py| replace:: tensor_grid.py
.. _tensor_grid.py: https://github.com/marcomusy/vedo/tree/master/examples/volumetric/tensor_grid.py

.. |scalarbars.py| replace:: scalarbars.py
.. _scalarbars.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/scalarbars.py
.. |scalarbars| image:: https://user-images.githubusercontent.com/32848391/62940174-4bdc7900-bdd3-11e9-9713-e4f3e2fdab63.png
    :width: 200 px
    :target: scalarbars.py_
    :alt: scalarbars.py

.. |erode_dilate.py| replace:: erode_dilate.py
.. _erode_dilate.py: https://github.com/marcomusy/vedo/tree/master/examples/volumetric/erode_dilate.py

.. |vol2points.py| replace:: vol2points.py
.. _vol2points.py: https://github.com/marcomusy/vedo/tree/master/examples/volumetric/vol2points.py

.. |euclDist.py| replace:: euclDist.py
.. _euclDist.py: https://github.com/marcomusy/vedo/tree/master/examples/volumetric/euclDist.py

.. |volumeFromMesh.py| replace:: volumeFromMesh.py
.. _volumeFromMesh.py: https://github.com/marcomusy/vedo/tree/master/examples/volumetric/volumeFromMesh.py

.. |numpy2volume1.py| replace:: numpy2volume1.py
.. _numpy2volume1.py: https://github.com/marcomusy/vedo/tree/master/examples/volumetric/numpy2volume1.py

.. |G_Of_Omega| image:: https://wikimedia.org/api/rest_v1/media/math/render/svg/9c4d02a66b6ff279aae0c4bf07c25e5727d192e4

.. |wikiphong| image:: https://upload.wikimedia.org/wikipedia/commons/6/6b/Phong_components_version_4.png

.. |animation1.py| replace:: animation1.py
.. _animation1.py: https://github.com/marcomusy/vedo/tree/master/examples/other/animation1.py
.. |animation1| image:: https://user-images.githubusercontent.com/32848391/64273764-4b528080-cf42-11e9-90aa-2d88df239871.gif
    :width: 200 px
    :target: animation1.py_
    :alt: animation1.py

.. |animation2.py| replace:: animation2.py
.. _animation2.py: https://github.com/marcomusy/vedo/tree/master/examples/other/animation2.py
.. |animation2| image:: https://user-images.githubusercontent.com/32848391/64273191-1a258080-cf41-11e9-8a18-f192f05f11a9.gif
    :width: 200 px
    :target: animation2.py_
    :alt: animation2.py

.. |polarHisto.py| replace:: polarHisto.py
.. _polarHisto.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/polarHisto.py
.. |polarHisto| image:: https://user-images.githubusercontent.com/32848391/64912717-5754f400-d733-11e9-8a1f-612165955f23.png
    :width: 200 px
    :target: polarHisto.py_
    :alt: polarHisto.py

.. |histo_polar.py| replace:: histo_polar.py
.. _histo_polar.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/histo_polar.py
.. |histo_polar| image:: https://user-images.githubusercontent.com/32848391/64992590-7fc82400-d8d4-11e9-9c10-795f4756a73f.png
    :width: 200 px
    :target: histo_polar.py_
    :alt: histo_polar.py

.. |donut.py| replace:: donut.py
.. _donut.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/donut.py
.. |donut| image:: https://user-images.githubusercontent.com/32848391/64998178-6f6b7580-d8e3-11e9-9bd8-8dfb9ccd90e4.png
    :width: 200 px
    :target: donut.py_
    :alt: donut.py

.. |extrude.py| replace:: extrude.py
.. _extrude.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/extrude.py
.. |extrude| image:: https://user-images.githubusercontent.com/32848391/65963682-971e1a00-e45b-11e9-9f29-05522ae4a800.png
    :width: 200 px
    :target: extrude.py_
    :alt: extrude.py

.. |kspline| image:: https://user-images.githubusercontent.com/32848391/65975805-73fd6580-e46f-11e9-8957-75eddb28fa72.png
    :width: 200 px

.. |mesh_lut.py| replace:: mesh_lut.py
.. _mesh_lut.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/mesh_lut.py

.. |elevation| image:: https://user-images.githubusercontent.com/32848391/68478872-3986a580-0231-11ea-8245-b68a683aa295.png
    :width: 200 px

.. |paramshapes| image:: https://user-images.githubusercontent.com/32848391/69181075-bb6aae80-0b0e-11ea-92f7-d0cd3b9087bf.png
    :width: 400 px

.. |warpto| image:: https://user-images.githubusercontent.com/32848391/69259878-3c817e80-0bbf-11ea-9025-03b9f6affccc.png
    :width: 200 px

.. |linInterpolate.py| replace:: linInterpolate.py
.. _linInterpolate.py: https://github.com/marcomusy/vedo/tree/master/examples/basic/linInterpolate.py
.. |linInterpolate| image:: https://user-images.githubusercontent.com/32848391/70559826-a621f680-1b87-11ea-89f3-e6b74d8953d9.png
    :width: 200 px
    :target: linInterpolate.py_
    :alt: linInterpolate.py

.. |plot_errbars.py| replace:: plot_errbars.py
.. _plot_errbars.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/plot_errbars.py
.. |plot_errbars| image:: https://user-images.githubusercontent.com/32848391/69158509-d6c1c380-0ae6-11ea-9dbf-ff5cd396a9a6.png
    :width: 200 px
    :target: plot_errbars.py_
    :alt: plot_errbars.py

.. |quiver.py| replace:: quiver.py
.. _quiver.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/quiver.py
.. |quiver| image::  https://user-images.githubusercontent.com/32848391/72261438-199aa600-3615-11ea-870e-e44ca4c4b8d3.png
    :width: 200 px
    :target: quiver.py_
    :alt: quiver.py

.. |plot_spheric.py| replace:: plot_spheric.py
.. _plot_spheric.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/plot_spheric.py
.. |plot_spheric| image:: https://user-images.githubusercontent.com/32848391/72433091-f0a31e00-3798-11ea-86bd-6c522e23ec61.png
    :width: 200 px
    :target: plot_spheric.py_
    :alt: plot_spheric.py

.. |fcomplex| image:: https://user-images.githubusercontent.com/32848391/73392962-1709a300-42db-11ea-9278-30c9d6e5eeaa.png
    :width: 200 px

.. |histo_spheric.py| replace:: histo_spheric.py
.. _histo_spheric.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/histo_spheric.py
.. |histo_spheric| image:: https://user-images.githubusercontent.com/32848391/73392901-fccfc500-42da-11ea-828a-9bad6982a823.png
    :width: 200 px
    :target: histo_spheric.py_
    :alt: histo_spheric.py

.. |sphericgrid| image:: https://user-images.githubusercontent.com/32848391/72433092-f0a31e00-3798-11ea-85f7-b2f5fcc31568.png
    :width: 200 px

.. |histo_2D.py| replace:: histo_2D.py
.. _histo_2D.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/histo_2D.py
.. |histo_2D| image:: https://user-images.githubusercontent.com/32848391/74361190-c019c880-4dc6-11ea-9c72-0f2a890e6664.png
    :width: 200 px
    :target: histo_2D.py_
    :alt: histo_2D.py

.. |plot_errband.py| replace:: plot_errband.py
.. _plot_errband.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/plot_errband.py
.. |plot_errband| image:: https://user-images.githubusercontent.com/32848391/73483464-c019d180-439f-11ea-9a8c-59fa49e9ecf4.png
    :width: 200 px
    :target: plot_errband.py_
    :alt: plot_errband.py

.. |plot_pip.py| replace:: plot_pip.py
.. _plot_pip.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/plot_pip.py
.. |plot_pip| image:: https://user-images.githubusercontent.com/32848391/73393632-4ff64780-42dc-11ea-8798-45a81c067f45.png
    :width: 200 px
    :target: plot_pip.py_
    :alt: plot_pip.py

.. |scatter1.py| replace:: scatter1.py
.. _scatter1.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/scatter1.py
.. |scatter1| image:: https://user-images.githubusercontent.com/32848391/72615028-013bcb80-3934-11ea-8ab8-823f1916bc6c.png
    :width: 200 px
    :target: scatter1.py_
    :alt: scatter1.py

.. |scatter2.py| replace:: scatter2.py
.. _scatter2.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/scatter2.py
.. |scatter2| image:: https://user-images.githubusercontent.com/32848391/72446102-2d7c0e80-37b3-11ea-8fe4-b27526af574f.png
    :width: 200 px
    :target: scatter2.py_
    :alt: scatter2.py

.. |scatter3.py| replace:: scatter3.py
.. _scatter3.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/scatter3.py
.. |scatter3| image:: https://user-images.githubusercontent.com/32848391/72446102-2d7c0e80-37b3-11ea-8fe4-b27526af574f.png
    :width: 200 px
    :target: scatter3.py_
    :alt: scatter3.py

.. |customIndividualAxes.py| replace:: customIndividualAxes.py
.. _customIndividualAxes.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/customIndividualAxes.py
.. |customIndividualAxes| image:: https://user-images.githubusercontent.com/32848391/72752870-ab7d5280-3bc3-11ea-8911-9ace00211e23.png
    :width: 200 px
    :target: customIndividualAxes.py_
    :alt: customIndividualAxes.py

.. |plot_stream.py| replace:: plot_stream.py
.. _plot_stream.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/plot_stream.py
.. |plot_stream| image:: https://user-images.githubusercontent.com/32848391/73614123-93162a80-45fc-11ea-969b-9a3293b26f35.png
    :width: 250 px
    :target: plot_stream.py_
    :alt: plot_stream.py

.. |simpleplot| image:: https://user-images.githubusercontent.com/32848391/74363882-c3638300-4dcb-11ea-8a78-eb492ad9711f.png
    :width: 200 px

.. |warpv| image:: https://user-images.githubusercontent.com/32848391/77864546-7a577900-7229-11ea-84ce-4e8e6eeff27f.png
    :width: 200 px

.. |lineage_graph.py| replace:: lineage_graph.py
.. _lineage_graph.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/graph_lineage.py
.. |lineage_graph| image:: https://user-images.githubusercontent.com/32848391/80291851-8152a800-8751-11ea-893e-4a0bb85397b1.png
    :width: 200 px
    :target: lineage_graph.py_
    :alt: lineage_graph.py

.. |bezier| image:: https://user-images.githubusercontent.com/32848391/90437534-dafd2a80-e0d2-11ea-9b93-9ecb3f48a3ff.png
    :width: 200 px

.. |goniometer.py| replace:: goniometer.py
.. _goniometer.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/goniometer.py
.. |goniometer| image:: https://user-images.githubusercontent.com/32848391/90437544-dd5f8480-e0d2-11ea-8321-b52d073444c4.png
    :width: 200 px
    :target: goniometer.py_
    :alt: goniometer.py

.. |intersect2d.py| replace:: intersect2d.py
.. _intersect2d.py: https://github.com/marcomusy/vedo/tree/master/examples/advanced/intersect2d.py
.. |intersect2d| image:: https://user-images.githubusercontent.com/32848391/90437548-de90b180-e0d2-11ea-8e0c-d821db4da8a9.png
    :width: 200 px
    :target: intersect2d.py_
    :alt: intersect2d.py

.. |fonts.py| replace:: fonts.py
.. _fonts.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/fonts3d.py
.. |fonts| image:: https://user-images.githubusercontent.com/32848391/90437539-dcc6ee00-e0d2-11ea-8381-93d211b1bc85.png
    :width: 400 px
    :target: fonts.py_
    :alt: fonts.py

.. |graph_network.py| replace:: graph_network.py
.. _graph_network.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/graph_network.py
.. |graph_network| image:: https://user-images.githubusercontent.com/32848391/90437546-ddf81b00-e0d2-11ea-84d5-e4356a5c5f85.png
    :width: 200 px
    :target: graph_network.py_
    :alt: graph_network.py

.. |plot_density3d.py| replace:: plot_density3d.py
.. _plot_density3d.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/plot_density3d.py
.. |plot_density3d| image:: https://user-images.githubusercontent.com/32848391/90437537-dc2e5780-e0d2-11ea-982c-8dafd467c3cd.png
    :width: 200 px
    :target: plot_density3d.py_
    :alt: plot_density3d.py

.. |fonts3d| image:: https://user-images.githubusercontent.com/32848391/90437540-dd5f8480-e0d2-11ea-8ddc-8839688979d0.png
    :width: 200 px

.. |fontlist| image:: https://user-images.githubusercontent.com/32848391/90437539-dcc6ee00-e0d2-11ea-8381-93d211b1bc85.png
    :width: 200 px

.. |caption.py| replace:: captions.py
.. _caption.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/caption.py
.. |caption| image:: https://user-images.githubusercontent.com/32848391/90437536-dc2e5780-e0d2-11ea-8951-f905ffb54f54.png
    :width: 200 px
    :target: caption.py_
    :alt: caption.py

.. |flag_labels.py| replace:: flag_labels.py
.. _flag_labels.py: https://github.com/marcomusy/vedo/tree/master/examples/other/flag_labels.py
.. |flag_labels| image:: https://user-images.githubusercontent.com/32848391/90620799-3b938100-e213-11ea-80b1-e05ce2949d3a.png
    :width: 200 px
    :target: flag_labels.py_
    :alt: flag_labels.py

.. |whiskers.py| replace:: whiskers.py
.. _whiskers.py: https://github.com/marcomusy/vedo/tree/master/examples/pyplot/whiskers.py
.. |whiskers| image:: https://user-images.githubusercontent.com/32848391/95772479-170cd000-0cbd-11eb-98c4-20c5ca342cb8.png
    :width: 200 px
    :target: whiskers.py_
    :alt: whiskers.py
"""




"""Use sliders to slice volume
Click button to change colormap"""
from vedo import dataurl, Volume, show, Text2D
from vedo.applications import SlicerPlotter

filename = dataurl+'embryo.slc'
#filename = dataurl+'embryo.tif'
#filename = dataurl+'vase.vti'

vol = Volume(filename)#.print()

plt = SlicerPlotter( vol,
                     bg='white', bg2='lightblue',
                     cmaps=("gist_ncar_r","jet","Spectral_r","hot_r","bone_r"),
                     useSlider3D=False,
                   )

#Can now add any other object to the Plotter scene:
#plt += Text2D('some message', font='arial')

plt.show().close()


# A plot(mode="bars") example. Useful to plot categories.
from vedo import precision, Text3D, colorMap, settings
from vedo.pyplot import plot

settings.defaultFont = "Meson"

counts  = [1946, 8993, 3042, 1190, 1477,    0,    0]
percent = [11.68909178, 54.01850072, 18.27246516,  7.14800577,  8.87193657, 0, 0]
labels  = ['<100', '100-250', '250-500', '500-750', '750-1000', '1000-2000', '>2000']
colors  = colorMap(range(len(counts)), "hot")

plt = plot([counts, labels, colors],
            mode="bars",
            ylim=(0,10500),
            aspect=4/3,
            axes=dict(htitle="Clusters in lux range",
                      hTitleItalic=False,
                      xLabelRotation=35,
                      xLabelSize=0.02,
                      tipSize=0, # axes arrow tip size
                     ),
)

for i in range(len(percent)):
    val = precision(percent[i], 3)+'%'
    txt = Text3D(val, pos=(plt.centers[i], counts[i]), justify="bottom-center", c="blue2")
    plt += txt.scale(200).shift(0,150,0)

plt.show(size=(1000,750), zoom=1.3, viewup='2d').close()



"""Mesh smoothing with two different methods"""
from vedo import Plotter, dataurl

plt = Plotter(N=2)

# Load a mesh and show it
vol = plt.load(dataurl+"embryo.tif")
m0 = vol.isosurface().normalize().lw(0.1).c("violet")
plt.show(m0, __doc__+"\nOriginal Mesh:", at=0)
plt.background([0.8, 1, 1], at=0)  # set first renderer color

# Smooth the mesh
m1 = m0.clone().smooth(niter=20).color("lg")

plt.show(m1, "Polygons are smoothed:", at=1, 
         viewup='z', zoom=1.5, interactive=True).close()


"""
General settings.

.. code-block:: python

    # Axes title defaults
    xtitle = 'x'
    ytitle = 'y'
    ztitle = 'z'

    # Set a default for the font to be used for axes, comments etc.
    defaultFont = 'Normografo' # check font options in shapes.Text

    # Scale magnification of the screenshot (must be an integer)
    screeshotScale = 1
    screenshotTransparentBackground = False
    screeshotLargeImage = False # Sometimes setting this to True gives better results

    # Recompute vertex and cell normals
    computeNormals = None

    # Allow to continously interact with scene during interactive() execution
    allowInteraction = True

    # Set up default mouse and keyboard functionalities
    enableDefaultMouseCallbacks = True
    enableDefaultKeyboardCallbacks = True

    # If False, when multiple renderers are present do not render each one for separate
    #  but do it just once at the end (when interactive() is called)
    immediateRendering = True

    # Show a gray frame margin in multirendering windows
    rendererFrameColor = None
    rendererFrameAlpha = 0.5
    rendererFrameWidth = 0.5
    rendererFramePadding = 0.0001

    # In multirendering mode set the position of the horizontal of vertical splitting [0,1]
    windowSplittingPosition = None

    # Enable / disable color printing by printc()
    enablePrintColor = True

    # Wrap lines in tubes
    renderLinesAsTubes = False

    # Smoothing options
    pointSmoothing = False
    lineSmoothing = False
    polygonSmoothing = False

    # Remove hidden lines when in wireframe mode
    hiddenLineRemoval = False

    # For Structured and RectilinearGrid: show internal edges not only outline
    visibleGridEdges = False

    # Turn on/off the automatic repositioning of lights as the camera moves.
    lightFollowsCamera = False
    twoSidedLighting = True

    # Turn on/off rendering of translucent material with depth peeling technique.
    useDepthPeeling = False
    alphaBitPlanes  = True  # options only active if useDepthPeeling=True
    multiSamples    = 8     # force to not pick a framebuffer with a multisample buffer
    maxNumberOfPeels= 4     # maximum number of rendering passes
    occlusionRatio  = 0.0   # occlusion ratio, 0 = exact image.

    # Turn on/off nvidia FXAA post-process anti-aliasing, if supported.
    useFXAA = False         # either True or False

    # By default, the depth buffer is reset for each renderer. If True, use the existing depth buffer
    preserveDepthBuffer = False

    # Turn on/off Screen Space Ambient Occlusion (SSAO), darken some pixels to improve depth perception
    useSSAO         = False
    SSAORadius      = 0.5   # the SSAO hemisphere radius
    SSAOBias        = 0.01  # the bias when comparing samples
    SSAOKernelSize  = 32    # the number of samples
    SSAOBlur        = False # blurring of the ambient occlusion (helps for low samples nr)

    # Use a polygon/edges offset to possibly resolve conflicts in rendering
    usePolygonOffset    = False
    polygonOffsetFactor = 0.1
    polygonOffsetUnits  = 0.1

    # Interpolate scalars to render them smoothly
    interpolateScalarsBeforeMapping = True

    # Set parallel projection On or Off (place camera to infinity, no perspective effects)
    useParallelProjection = False

    # Set orientation type when reading TIFF files (volumes):
    # TOPLEFT  1 (row 0 top, col 0 lhs)    TOPRIGHT 2 (row 0 top, col 0 rhs)
    # BOTRIGHT 3 (row 0 bottom, col 0 rhs) BOTLEFT  4 (row 0 bottom, col 0 lhs)
    # LEFTTOP  5 (row 0 lhs, col 0 top)    RIGHTTOP 6 (row 0 rhs, col 0 top)
    # RIGHTBOT 7 (row 0 rhs, col 0 bottom) LEFTBOT  8 (row 0 lhs, col 0 bottom)
    tiffOrientationType = 1

    # AnnotatedCube axis type nr. 5 options:
    annotatedCubeColor      = (0.75, 0.75, 0.75)
    annotatedCubeTextColor  = None # use default, otherwise specify a single color
    annotatedCubeTextScale  = 0.2
    annotatedCubeTexts      = ["right","left ", "front","back ", " top ", "bttom"]

    # k3d settings for jupyter notebooks
    k3dMenuVisibility = True
    k3dPlotHeight = 512
    k3dAntialias  = True
    k3dLighting   = 1.2
    k3dCameraAutoFit = True
    k3dGridAutoFit= True
    k3dAxesHelper = True    # size of the small triad of axes on the bottom right
    k3dPointShader= "mesh"  # others are '3d', '3dSpecular', 'dot', 'flat'
    k3dLineShader = "thick" # others are 'flat', 'mesh'

Usage example:

.. code-block:: python

    from vedo import *

    settings.useParallelProjection = True

    Cube().color('green').show()
"""
import os, vtk
import numpy as np
import warnings

__all__ = ['dataurl', 'embedWindow']


vtk_version = [ int(vtk.vtkVersion().GetVTKMajorVersion()),
                int(vtk.vtkVersion().GetVTKMinorVersion()),
                int(vtk.vtkVersion().GetVTKBuildVersion()) ]

try:
    import platform
    sys_platform = platform.system()
except:
    sys_platform = ""


####################################################################################
# Axes titles
xtitle = 'x'
ytitle = 'y'
ztitle = 'z'
 
# Set a default for the font to be used for axes, comments etc.
defaultFont = 'Normografo'

# Scale magnification of the screenshot (must be an integer)
screeshotScale = 1
screenshotTransparentBackground = False
screeshotLargeImage = False

# Recompute vertex and cell normals
computeNormals = None

# Allow to continously interact with scene during interactor.Start() execution
allowInteraction = True

# Set up default mouse and keyboard functionalities
enableDefaultMouseCallbacks = True
enableDefaultKeyboardCallbacks = True

# When multiple renderers are present do not render each one for separate.
# but do it just once at the end (when interactive() is called)
immediateRendering = True

# Show a gray frame margin in multirendering windows
rendererFrameColor = None
rendererFrameAlpha = 0.5
rendererFrameWidth = 0.5
rendererFramePadding = 0.0001

# Wrap lines in tubes
# renderPointsAsSpheres has become mesh.renderPointsAsSpheres(True)
renderLinesAsTubes = False

# Remove hidden lines when in wireframe mode
hiddenLineRemoval = False

# Smoothing options
pointSmoothing = False
lineSmoothing = False
polygonSmoothing = False

# For Structured and RectilinearGrid: show internal edges not only outline
visibleGridEdges = False

# Turn on/off the automatic repositioning of lights as the camera moves.
lightFollowsCamera = False
twoSidedLighting = True

# Turn on/off rendering of translucent material with depth peeling technique.
#print("vtk_version sys_platform", vtk_version, sys_platform)
useDepthPeeling = False
multiSamples = 8
if vtk_version[0] >= 9:
    if "Windows" in sys_platform:
        useDepthPeeling = True
# only relevant if depthpeeling is on
alphaBitPlanes   = 1
maxNumberOfPeels = 4
occlusionRatio   = 0.1

# Turn on/off nvidia FXAA anti-aliasing, if supported.
useFXAA = False  # either True or False

# By default, the depth buffer is reset for each renderer. If true, use the existing depth buffer
preserveDepthBuffer = False

#Enable or disable Screen Space Ambient Occlusion: SSAO darkens some pixels to improve depth perception.
useSSAO        = False
SSAORadius     = 0.5     # define the SSAO hemisphere radius
SSAOBias       = 0.01    # define the bias when comparing samples
SSAOKernelSize = 32      # define the number of samples
SSAOBlur       = False   # define blurring of the ambient occlusion (helps for low samples)

# Use a polygon/edges offset to possibly resolve conflicts in rendering
usePolygonOffset = False
polygonOffsetFactor = 0.1
polygonOffsetUnits  = 0.1

# Interpolate scalars to render them smoothly
interpolateScalarsBeforeMapping = True

# Set parallel projection On or Off (place camera to infinity, no perspective effects)
useParallelProjection = False

# In multirendering mode set the position of the horizontal of vertical splitting [0,1]
windowSplittingPosition = None

# Set orientation type when reading TIFF files (volumes):
# TOPLEFT  1 (row 0 top, col 0 lhs)    TOPRIGHT 2 (row 0 top, col 0 rhs)
# BOTRIGHT 3 (row 0 bottom, col 0 rhs) BOTLEFT  4 (row 0 bottom, col 0 lhs)
# LEFTTOP  5 (row 0 lhs, col 0 top)    RIGHTTOP 6 (row 0 rhs, col 0 top)
# RIGHTBOT 7 (row 0 rhs, col 0 bottom) LEFTBOT  8 (row 0 lhs, col 0 bottom)
tiffOrientationType = 1

# AnnotatedCube axis (type 5) customization:
annotatedCubeColor      = (0.75, 0.75, 0.75)
annotatedCubeTextColor  = None # use default, otherwise specify a single color
annotatedCubeTextScale  = 0.2
annotatedCubeTexts      = ["right","left ", "front","back ", " top ", "bttom"]

# enable / disable color printing
enablePrintColor = True

####################################################################################
# notebook support
notebookBackend = None
notebook_plotter = None

# k3d settings for jupyter notebooks
k3dMenuVisibility = True
k3dPlotHeight = 512
k3dAntialias  = True
k3dLighting   = 1.2
k3dCameraAutoFit = True
k3dGridAutoFit= True
k3dAxesHelper = True    # size of the small triad of axes on the bottom right
k3dPointShader= "mesh"  # others are '3d', '3dSpecular', 'dot', 'flat'
k3dLineShader = "thick" # others are 'flat', 'mesh'

####################################################################################
flagDelay = 150 # values will be superseded
flagFont = "Courier"
flagFontSize = 18
flagJustification = 0
flagAngle = 0
flagBold = False
flagItalic = False
flagShadow = False
flagColor = 'k'
flagBackgroundColor = 'w'


#######################################################################################
#######################################################################################
installdir = os.path.dirname(__file__)

textures_path = os.path.join(installdir, "textures/")
textures = []

fonts_path = os.path.join(installdir, "fonts/")
fonts = []

#dataurl = "/home/musy/Dropbox/Public/vktwork/vedo_data/"; print('\ndataurl=',dataurl)
dataurl = "https://vedo.embl.es/examples/data/"

plotter_instances = []
plotter_instance = None

interactorStyle = None # internal use only

####################################################################################
####################################################################################
# mono       # means that all letters occupy the same space slot horizontally
# hspacing   # an horizontal stretching factor (affects both letters and words)
# lspacing   # horizontal spacing inbetween letters (not words)
# islocal    # is locally stored in /fonts, otherwise it's on vedo.embl.es/fonts

font_parameters = dict(

        Normografo = dict(
                        mono = False,
                        fscale = 0.75,
                        hspacing = 1,
                        lspacing = 0.2,
                        dotsep = "~·",
                        islocal = True,
                        ),
        Bongas = dict(
                        mono = False,
                        fscale = 0.875,
                        hspacing = 0.52,
                        lspacing = 0.25,
                        dotsep = "·",
                        islocal = True,
                        ),
        Calco = dict(
                        mono = True,
                        fscale = 0.8,
                        hspacing = 1,
                        lspacing = 0.1,
                        dotsep = "·",
                        islocal = True,
                        ),
        Comae = dict(
                        mono = False,
                        fscale = 0.75,
                        lspacing = 0.2,
                        hspacing = 1,
                        dotsep = '~·',
                        islocal = True,
                        ),
        Glasgo = dict(
                        mono = True,
                        fscale = 0.75,
                        lspacing = 0.1,
                        hspacing = 1,
                        dotsep = "·",
                        islocal = True,
                        ),
        Kanopus = dict(
                        mono = False,
                        fscale = 0.75,
                        lspacing = 0.15,
                        hspacing = 0.75,
                        dotsep = '~·',
                        islocal = True,
                        ),
        LionelOfParis = dict(
                        mono = False,
                        fscale = 0.875,
                        hspacing = 0.7,
                        lspacing = 0.3,
                        dotsep = "·",
                        islocal = True,
                        ),
        LogoType = dict(
                        mono = False,
                        fscale = 0.75,
                        hspacing = 1,
                        lspacing = 0.2,
                        dotsep = '·~~',
                        islocal = False,
                        ),
        Quikhand = dict(
                        mono = False,
                        fscale = 0.8,
                        hspacing = 0.6,
                        lspacing = 0.15,
                        dotsep = "~~·~",
                        islocal = True,
                        ),
        SmartCouric = dict(
                        mono = True,
                        fscale = 0.8,
                        hspacing = 1.05,
                        lspacing = 0.1,
                        dotsep = "·",
                        islocal = True,
                        ),
        Spears = dict(
                        mono = False,
                        fscale = 0.8,
                        hspacing = 0.5,
                        lspacing = 0.2,
                        dotsep = "·",
                        islocal = False,
                        ),
        Theemim = dict(
                        mono = False,
                        fscale = 0.825,
                        hspacing = 0.52,
                        lspacing = 0.3,
                        dotsep = '~·',
                        islocal = True,
                        ),
        VictorMono = dict(
                        mono = True,
                        fscale = 0.725,
                        hspacing = 1,
                        lspacing = 0.1,
                        dotsep = "·",
                        islocal = True,
                        ),
        Justino1 = dict(
                        mono = True,
                        fscale = 0.725,
                        hspacing = 1,
                        lspacing = 0.1,
                        dotsep = "·",
                        islocal = False,
                        ),
        Justino2 = dict(
                        mono = True,
                        fscale = 0.725,
                        hspacing = 1,
                        lspacing = 0.1,
                        dotsep = "·",
                        islocal = False,
                        ),
        Justino3 = dict(
                        mono = True,
                        fscale = 0.725,
                        hspacing = 1,
                        lspacing = 0.1,
                        dotsep = "·",
                        islocal = False,
                        ),
        Justino4 = dict(
                        mono = True,
                        fscale = 0.725,
                        hspacing = 1,
                        lspacing = 0.1,
                        dotsep = "·",
                        islocal = False,
                        ),
        Capsmall = dict(
                        mono = False,
                        fscale = 0.8,
                        hspacing = 0.75,
                        lspacing = 0.15,
                        dotsep = "·",
                        islocal = False,
                        ),
        Cartoons123 = dict(
                        mono = False,
                        fscale = 0.8,
                        hspacing = 0.75,
                        lspacing = 0.15,
                        dotsep = "·",
                        islocal = False,
                        ),
        Vega = dict(
                        mono = False,
                        fscale = 0.8,
                        hspacing = 0.75,
                        lspacing = 0.15,
                        dotsep = "·",
                        islocal = False,
                        ),
        PlanetBenson = dict(
                        mono = False,
                        fscale = 0.8,
                        hspacing = 0.8,
                        lspacing = 0.11,
                        dotsep = "·",
                        islocal = False,
                        ),
        Meson= dict(
                        mono = False,
                        fscale = 0.8,
                        hspacing = 0.9,
                        lspacing = 0.225,
                        dotsep = "~^.~ ",
                        islocal = False,
                        ),
        Komika= dict(
                        mono = False,
                        fscale = 0.7,
                        hspacing = 0.75,
                        lspacing = 0.225,
                        dotsep = "~^.~ ",
                        islocal = False,
                        ),
)


####################################################################################
def embedWindow(backend='ipyvtk', verbose=True):
    """Use this function to control whether the rendering window is inside
    the jupyter notebook or as an independent external window"""
    global notebook_plotter, notebookBackend

    if not backend:
        notebookBackend = None
        notebook_plotter = None
        return
    else:

        if any(['SPYDER' in name for name in os.environ]):
            notebookBackend = None
            notebook_plotter = None
            return

        try:
            get_ipython()
        except:
            notebookBackend = None
            notebook_plotter = None
            return

    backend = backend.lower()
    notebookBackend = backend

    if backend=='k3d':
        try:
            import k3d
            if k3d._version.version_info != (2, 7, 4):
                print('Warning: only k3d version 2.7.4 is currently supported')
                # print('> pip install k3d==2.7.4')
            
        except:
            notebookBackend = None
            if verbose:
                print('embedWindow(verbose=True): could not load k3d module, try:')
                print('> pip install k3d==2.7.4')

    elif 'ipygany' in backend: # ipygany
        try:
            import ipygany
        except:
            notebookBackend = None
            if verbose:
                print('embedWindow(verbose=True): could not load ipygany module, try:')
                print('> pip install ipygany')

    elif 'itk' in backend: # itkwidgets
        try:
            import itkwidgets
        except:
            notebookBackend = None
            if verbose:
                print('embedWindow(verbose=True): could not load itkwidgets module, try:')
                print('> pip install itkwidgets')

    elif backend.lower() == '2d':
        pass

    elif backend =='panel':
        try:
            import panel
            panel.extension('vtk')
        except:
            if verbose:
                print('embedWindow(verbose=True): could not load panel try:')
                print('> pip install panel')

    elif 'ipyvtk' in backend:
        try:
            from ipyvtklink.viewer import ViewInteractiveWidget
        except:
            if verbose:
                print('embedWindow(verbose=True): could not load ipyvtklink try:')
                print('> pip install ipyvtklink')

    else:
        print("Unknown backend", backend)
        raise RuntimeError()


#####################
def _init():
    global plotter_instance, plotter_instances
    global textures, fonts
    global notebookBackend, notebook_plotter

    plotter_instance = None
    plotter_instances = []

    for f in os.listdir(textures_path):
        tfn = f.split(".")[0]
        if 'earth' in tfn: continue
        textures.append(tfn)

    for f in os.listdir(fonts_path):
        if '.npz' in f: continue
        fonts.append(f.split(".")[0])
    fonts = list(sorted(fonts))

    warnings.simplefilter(action="ignore", category=FutureWarning)
    np.warnings.filterwarnings('ignore', category=np.VisibleDeprecationWarning) 

    embedWindow()



import numpy as np
import vtk
import vedo
import vedo.colors as colors
import vedo.utils as utils
from vtk.util.numpy_support import numpy_to_vtk

__doc__ = (
    """
Submodule extending the ``vtkImageActor`` object functionality.
"""
    + vedo.docs._defs
)

__all__ = ["Picture"]


#################################################
class Picture(vtk.vtkImageActor, vedo.base.Base3DProp):   
    """
    Derived class of ``vtkImageActor``. Used to represent 2D pictures.
    Can be instantiated with a path file name or with a numpy array.

    Use `Picture.shape` to access the number of pixels in x and y.

    |rotateImage| |rotateImage.py|_
    
    :param list channels: only select these specific rgba channels (useful to remove alpha)
    :param bool flip: flip xy axis convention (when input is a numpy array)
    """    
    def __init__(self, obj=None, channels=(), flip=False):
        vtk.vtkImageActor.__init__(self)
        vedo.base.Base3DProp.__init__(self)


        if utils.isSequence(obj) and len(obj): # passing array
            obj = np.asarray(obj)

            if len(obj.shape) == 3: # has shape (nx,ny, ncolor_alpha_chan)
                iac = vtk.vtkImageAppendComponents()
                nchan = obj.shape[2] # get number of channels in inputimage (L/LA/RGB/RGBA)
                for i in range(nchan):
                    if flip:
                        arr = np.flip(np.flip(obj[:,:,i], 0), 0).ravel()
                    else:
                        arr = np.flip(obj[:,:,i], 0).ravel()
                    varb = numpy_to_vtk(arr, deep=True, array_type=vtk.VTK_UNSIGNED_CHAR)
                    varb.SetName("RGBA")
                    imgb = vtk.vtkImageData()
                    imgb.SetDimensions(obj.shape[1], obj.shape[0], 1)
                    imgb.GetPointData().AddArray(varb)
                    imgb.GetPointData().SetActiveScalars("RGBA")
                    iac.AddInputData(imgb)
                iac.Update()
                img = iac.GetOutput()

            elif len(obj.shape) == 2: # black and white
                if flip:
                    arr = np.flip(obj[:,:], 0).ravel()
                else:
                    arr = obj.ravel()
                varb = numpy_to_vtk(arr, deep=True, array_type=vtk.VTK_UNSIGNED_CHAR)
                varb.SetName("RGBA")
                img = vtk.vtkImageData()
                img.SetDimensions(obj.shape[1], obj.shape[0], 1)
                img.GetPointData().AddArray(varb)
                img.GetPointData().SetActiveScalars("RGBA")

        elif isinstance(obj, vtk.vtkImageData):
            img = obj

        elif isinstance(obj, str):
            if "https://" in obj:
                obj = vedo.io.download(obj, verbose=False)

            fname = obj.lower()
            if fname.endswith(".png"):
                picr = vtk.vtkPNGReader()
            elif fname.endswith(".jpg") or fname.endswith(".jpeg"):
                picr = vtk.vtkJPEGReader()
            elif fname.endswith(".bmp"):
                picr = vtk.vtkBMPReader()
            elif fname.endswith(".tif") or fname.endswith(".tiff"):
                picr = vtk.vtkTIFFReader()
                picr.SetOrientationType(vedo.settings.tiffOrientationType)
            else:
                colors.printc("Cannot understand picture format", obj, c='r')
                return
            picr.SetFileName(obj)
            self.filename = obj
            picr.Update()
            img = picr.GetOutput()

        else:
            img = vtk.vtkImageData()

        # select channels
        nchans = len(channels)
        if nchans and img.GetPointData().GetScalars().GetNumberOfComponents() > nchans:
            pec = vtk.vtkImageExtractComponents()
            pec.SetInputData(img)
            if nchans == 3:
                pec.SetComponents(channels[0], channels[1], channels[2])
            elif nchans == 2:
                pec.SetComponents(channels[0], channels[1])
            elif nchans == 1:
                pec.SetComponents(channels[0])
            pec.Update()
            img = pec.GetOutput()

        self._data = img
        self.SetInputData(img)

        sx,sy,_ = img.GetDimensions()
        self.shape = np.array([sx,sy])

        self._mapper = self.GetMapper()


    def inputdata(self):
        """Return the underlying ``vtkImagaData`` object."""
        return self._data
    
    def dimensions(self):
        nx, ny, _ = self._data.GetDimensions()
        return np.array([nx, ny])

    def _update(self, data):
        """Overwrite the Picture data mesh with a new data."""
        self._data = data
        self._mapper.SetInputData(data)
        self._mapper.Modified()
        return self

    def clone(self, transform=False):
        """Return an exact copy of the input Picture.
        If transform is True, it is given the same scaling and position."""
        img = vtk.vtkImageData()
        img.DeepCopy(self._data)
        pic = Picture(img)
        if transform:
            # assign the same transformation to the copy
            pic.SetOrigin(self.GetOrigin())
            pic.SetScale(self.GetScale())
            pic.SetOrientation(self.GetOrientation())
            pic.SetPosition(self.GetPosition())
        return pic

    def extent(self, ext=None):
        """
        Get or set the physical extent that the picture spans.
        Format is ext=[minx, maxx, miny, maxy].
        """
        if ext is None:
            return self._data.GetExtent()
        self._data.SetExtent(ext[0],ext[1],ext[2],ext[3],0,0)
        self._mapper.Modified()
        return self
    
    def text(self, txt,
                   pos=(0,0,0),
                   s=1,
                   c=None,
                   alpha=1,
                   bg=None,
                   font="Theemim",
                   dpi=500,
                   justify="bottom-left",
        ):
        """Build an image from a string."""

        if c is None: # automatic black or white
            if vedo.settings.plotter_instance and vedo.settings.plotter_instance.renderer:
                c = (0.9, 0.9, 0.9)
                if np.sum(vedo.settings.plotter_instance.renderer.GetBackground()) > 1.5:
                    c = (0.1, 0.1, 0.1)
            else:
                c = (0.3, 0.3, 0.3)

        r = vtk.vtkTextRenderer()
        img = vtk.vtkImageData()

        tp = vtk.vtkTextProperty()
        tp.BoldOff()
        tp.SetColor(colors.getColor(c))
        tp.SetJustificationToLeft()
        if "top" in justify:
            tp.SetVerticalJustificationToTop()
        if "bottom" in justify:
            tp.SetVerticalJustificationToBottom()
        if "cent" in justify:
            tp.SetVerticalJustificationToCentered()
            tp.SetJustificationToCentered()
        if "left" in justify:
            tp.SetJustificationToLeft()
        if "right" in justify:
            tp.SetJustificationToRight()

        if font.lower() == "courier": tp.SetFontFamilyToCourier()
        elif font.lower() == "times": tp.SetFontFamilyToTimes()
        elif font.lower() == "arial": tp.SetFontFamilyToArial()
        else:
            tp.SetFontFamily(vtk.VTK_FONT_FILE)
            tp.SetFontFile(utils.getFontPath(font))

        if bg:
            bgcol = colors.getColor(bg)
            tp.SetBackgroundColor(bgcol)
            tp.SetBackgroundOpacity(alpha * 0.5)
            tp.SetFrameColor(bgcol)
            tp.FrameOn()

        #GetConstrainedFontSize (const vtkUnicodeString &str,
        # vtkTextProperty *tprop, int targetWidth, int targetHeight, int dpi)
        fs = r.GetConstrainedFontSize(txt, tp, 900, 1000, dpi)
        tp.SetFontSize(fs)

        r.RenderString(tp, txt, img, [1,1], dpi)
        # RenderString (vtkTextProperty *tprop, const vtkStdString &str,
        #   vtkImageData *data, int textDims[2], int dpi, int backend=Default)

        self.SetInputData(img)
        self.GetMapper().Modified()

        self.SetPosition(pos)
        x0, x1 = self.xbounds()
        if x1 != x0:
            sc = s/(x1-x0)
            self.SetScale(sc,sc,sc)
        return self


    def alpha(self, a=None):
        """Set/get picture's transparency."""
        if a is not None:
            self.GetProperty().SetOpacity(a)
            return self
        else:
            return self.GetProperty().GetOpacity()

    def level(self, value=None):
        """Get/Set the image color level (brightness)."""
        if value is None:
            return self.GetProperty().GetColorLevel()
        self.GetProperty().SetColorLevel(value)
        return self

    def window(self, value=None):
        """Get/Set the image color window (contrast)."""
        if value is None:
            return self.GetProperty().GetColorWindow()
        self.GetProperty().SetColorWindow(value)
        return self


    def crop(self, top=None, bottom=None, right=None, left=None, pixels=False):
        """Crop picture.

        :param float top: fraction to crop from the top margin
        :param float bottom: fraction to crop from the bottom margin
        :param float left: fraction to crop from the left margin
        :param float right: fraction to crop from the right margin
        :param bool pixels: units are pixels
        """
        extractVOI = vtk.vtkExtractVOI()
        extractVOI.SetInputData(self._data)
        extractVOI.IncludeBoundaryOn()

        d = self.GetInput().GetDimensions()
        if pixels:
            extractVOI.SetVOI(right, d[0]-left, bottom, d[1]-top, 0, 0)
        else:
            bx0, bx1, by0, by1 = 0, d[0]-1, 0, d[1]-1
            if left is not None:   bx0 = int((d[0]-1)*left)
            if right is not None:  bx1 = int((d[0]-1)*(1-right))
            if bottom is not None: by0 = int((d[1]-1)*bottom)
            if top is not None:    by1 = int((d[1]-1)*(1-top))
            extractVOI.SetVOI(bx0, bx1, by0, by1, 0, 0)
        extractVOI.Update()
        return self._update(extractVOI.GetOutput())

    def pad(self, pixels=10, value=255):
        """
        Add the specified number of pixels at the picture borders.
        Pixels can be a list formatted as [left,right,bottom,top].
        
        Parameters
        ----------
        pixels : int,list , optional
            number of pixels to be added (or a list of length 4). The default is 10.
        value : int, optional
            intensity value (gray-scale color) of the padding. The default is 255.
        """
        x0,x1,y0,y1,_z0,_z1 = self._data.GetExtent()
        pf = vtk.vtkImageConstantPad()
        pf.SetInputData(self._data)
        pf.SetConstant(value)
        if utils.isSequence(pixels):
            pf.SetOutputWholeExtent(x0-pixels[0],x1+pixels[1],
                                    y0-pixels[2],y1+pixels[3], 0,0)
        else:
            pf.SetOutputWholeExtent(x0-pixels,x1+pixels, y0-pixels,y1+pixels, 0,0)
        pf.Update()
        img = pf.GetOutput()
        return self._update(img)


    def tiling(self, nx=4, ny=4, shift=(0,0)):
        """
        Generate a tiling from the current picture by mirroring and repeating it.

        Parameters
        ----------
        nx :  float, optional
            number of repeats along x. The default is 4.
        ny : float, optional
            number of repeats along x. The default is 4.
        shift : list, optional
            shift in x and y in pixels. The default is 4.
        """
        x0,x1,y0,y1,z0,z1 = self._data.GetExtent()
        constantPad = vtk.vtkImageMirrorPad()
        constantPad.SetInputData(self._data)
        constantPad.SetOutputWholeExtent(int(x0+shift[0]+0.5), int(x1*nx+shift[0]+0.5),
                                         int(y0+shift[1]+0.5), int(y1*ny+shift[1]+0.5), z0,z1)
        constantPad.Update()
        return Picture(constantPad.GetOutput())


    def append(self, pictures, axis='z', preserveExtents=False):
        """
        Append the input images to the current one along the specified axis.
        Except for the append axis, all inputs must have the same extent.
        All inputs must have the same number of scalar components.
        The output has the same origin and spacing as the first input.
        The origin and spacing of all other inputs are ignored.
        All inputs must have the same scalar type.

        :param int,str axis: axis expanded to hold the multiple images.
        :param bool preserveExtents: if True, the extent of the inputs is used to place
            the image in the output. The whole extent of the output is the union of the input
            whole extents. Any portion of the output not covered by the inputs is set to zero.
            The origin and spacing is taken from the first input.

        .. code-block:: python

            from vedo import Picture, dataurl
            pic = Picture(dataurl+'dog.jpg').pad()
            pic.append([pic,pic,pic], axis='y')
            pic.append([pic,pic,pic,pic], axis='x')
            pic.show(axes=1)
        """
        ima = vtk.vtkImageAppend()
        ima.SetInputData(self._data)
        if not utils.isSequence(pictures):
            pictures = [pictures]
        for p in pictures:
            if isinstance(p, vtk.vtkImageData):
                ima.AddInputData(p)
            else:
                ima.AddInputData(p._data)
        ima.SetPreserveExtents(preserveExtents)
        if axis   == "x":
            axis = 0
        elif axis == "y":
            axis = 1
        ima.SetAppendAxis(axis)
        ima.Update()
        return self._update(ima.GetOutput())
    

    def resize(self, newsize):
        """Resize the image resolution by specifying the number of pixels in width and height.
        If left to zero, it will be automatically calculated to keep the original aspect ratio.

        :param list,float newsize: shape of picture as [npx, npy], or as a fraction.
        """
        old_dims = np.array(self._data.GetDimensions())

        if not utils.isSequence(newsize):
            newsize = (old_dims * newsize + 0.5).astype(int)

        if not newsize[1]:
            ar = old_dims[1]/old_dims[0]
            newsize = [newsize[0], int(newsize[0]*ar+0.5)]
        if not newsize[0]:
            ar = old_dims[0]/old_dims[1]
            newsize = [int(newsize[1]*ar+0.5), newsize[1]]
        newsize = [newsize[0], newsize[1], old_dims[2]]

        rsz = vtk.vtkImageResize()
        rsz.SetInputData(self._data)
        rsz.SetResizeMethodToOutputDimensions()
        rsz.SetOutputDimensions(newsize)
        rsz.Update()
        out = rsz.GetOutput()
        out.SetSpacing(1,1,1)
        return self._update(out)

    def mirror(self, axis="x"):
        """Mirror picture along x or y axis."""
        ff = vtk.vtkImageFlip()
        ff.SetInputData(self.inputdata())
        if axis.lower() == "x":
            ff.SetFilteredAxis(0)
        elif axis.lower() == "y":
            ff.SetFilteredAxis(1)
        else:
            colors.printc("Error in mirror(): mirror must be set to x or y.", c='r')
            raise RuntimeError()
        ff.Update()
        return self._update(ff.GetOutput())
    
    def rotate(self, angle, center=(), scale=1, mirroring=False, bc='w', alpha=1):
        """
        Rotate an image by an angle (anticlockwise).

        Parameters
        ----------
        angle : float
            rotation angle in degrees.
        center: list
            center of rotation (x,y) in pixels.
        """        
        bounds = self.bounds()
        pc = [0,0,0]
        if center:
            pc[0] = center[0]
            pc[1] = center[1]
        else:
            pc[0] = (bounds[1] + bounds[0]) / 2.0
            pc[1] = (bounds[3] + bounds[2]) / 2.0
        pc[2] = (bounds[5] + bounds[4]) / 2.0
  
        transform = vtk.vtkTransform()
        transform.Translate(pc)
        transform.RotateWXYZ(-angle, 0, 0, 1)
        transform.Scale(1/scale,1/scale,1)
        transform.Translate(-pc[0], -pc[1], -pc[2])

        reslice = vtk.vtkImageReslice()
        reslice.SetMirror(mirroring)
        c = np.array(colors.getColor(bc))*255
        reslice.SetBackgroundColor([c[0],c[1],c[2], alpha*255])
        reslice.SetInputData(self._data)
        reslice.SetResliceTransform(transform)
        reslice.SetOutputDimensionality(2)
        reslice.SetInterpolationModeToCubic()
        reslice.SetOutputSpacing(self._data.GetSpacing())
        reslice.SetOutputOrigin(self._data.GetOrigin())
        reslice.SetOutputExtent(self._data.GetExtent())
        reslice.Update()
        return self._update(reslice.GetOutput())

    def select(self, component):
        """Select one single component of the rgb image"""
        ec = vtk.vtkImageExtractComponents()
        ec.SetInputData(self._data)
        ec.SetComponents(component)
        ec.Update()
        return Picture(ec.GetOutput())
    
    def bw(self):
        """Make it black and white"""
        n = self._data.GetPointData().GetNumberOfComponents()        
        if n==4:
            ecr = vtk.vtkImageExtractComponents()
            ecr.SetInputData(self._data)
            ecr.SetComponents(0,1,2)
            ecr.Update()
            img = ecr.GetOutput()
        else:
            img = self._data
        
        ecr = vtk.vtkImageLuminance()
        ecr.SetInputData(img)
        ecr.Update()
        return self._update(ecr.GetOutput())

    def smooth(self, sigma=3, radius=None):
        """
        Smooth a Picture with Gaussian kernel.

        Parameters
        ----------
        sigma : int, optional
            number of sigmas in pixel units. The default is 3.
        radius : TYPE, optional
            how far out the gaussian kernel will go before being clamped to zero. The default is None.
        """
        gsf = vtk.vtkImageGaussianSmooth()
        gsf.SetDimensionality(2)
        gsf.SetInputData(self._data)
        if radius is not None:
            if utils.isSequence(radius):
                gsf.SetRadiusFactors(radius[0],radius[1])
            else:
                gsf.SetRadiusFactor(radius)
        
        if utils.isSequence(sigma):
            gsf.SetStandardDeviations(sigma[0], sigma[1])
        else:
            gsf.SetStandardDeviation(sigma)
        gsf.Update()
        return self._update(gsf.GetOutput())

    def median(self):
        """Median filter that preserves thin lines and corners. 
        It operates on a 5x5 pixel neighborhood. It computes two values initially: 
        the median of the + neighbors and the median of the x neighbors. 
        It then computes the median of these two values plus the center pixel. 
        This result of this second median is the output pixel value.
        """
        medf = vtk.vtkImageHybridMedian2D()
        medf.SetInputData(self._data)
        medf.Update()        
        return self._update(medf.GetOutput())
    
    def enhance(self):
        """
        Enhance a b&w picture using the laplacian, enhancing high-freq edges.
        
        Example:
            
            .. code-block:: python
            
                import vedo
                p = vedo.Picture(vedo.dataurl+'images/dog.jpg').bw()
                vedo.show(p, p.clone().enhance(), N=2, mode='image')
        """       
        img = self._data
        scalarRange = img.GetPointData().GetScalars().GetRange()
        
        cast = vtk.vtkImageCast()
        cast.SetInputData(img)
        cast.SetOutputScalarTypeToDouble()
        cast.Update()
       
        laplacian = vtk.vtkImageLaplacian()
        laplacian.SetInputData(cast.GetOutput())
        laplacian.SetDimensionality(2)
        laplacian.Update()
        
        subtr = vtk.vtkImageMathematics()
        subtr.SetInputData(0, cast.GetOutput())
        subtr.SetInputData(1, laplacian.GetOutput())
        subtr.SetOperationToSubtract()
        subtr.Update()    
        
        colorWindow = scalarRange[1] - scalarRange[0]
        colorLevel = colorWindow / 2    
        originalColor = vtk.vtkImageMapToWindowLevelColors()
        originalColor.SetWindow(colorWindow)
        originalColor.SetLevel(colorLevel)
        originalColor.SetInputData(subtr.GetOutput())
        originalColor.Update()        
        return self._update(originalColor.GetOutput())

    def fft(self, mode='magnitude', logscale=12, center=True):
        """Fast Fourier transform of a picture.

        :param float logscale: if non-zero, take the logarithm of the
            intensity and scale it by this factor.

        :param str mode: either [magnitude, real, imaginary, complex], compute the
            point array data accordingly.
        :param bool center: shift constant zero-frequency to the center of the image for display.
            (FFT converts spatial images into frequency space, but puts the zero frequency at the origin)
        """
        ffti = vtk.vtkImageFFT()
        ffti.SetInputData(self._data)
        ffti.Update()

        if 'mag' in mode:
            mag = vtk.vtkImageMagnitude()
            mag.SetInputData(ffti.GetOutput())
            mag.Update()
            out = mag.GetOutput()
        elif 'real' in mode:
            extractRealFilter = vtk.vtkImageExtractComponents()
            extractRealFilter.SetInputData(ffti.GetOutput())
            extractRealFilter.SetComponents(0)
            extractRealFilter.Update()
            out = extractRealFilter.GetOutput()
        elif 'imaginary' in mode:
            extractImgFilter = vtk.vtkImageExtractComponents()
            extractImgFilter.SetInputData(ffti.GetOutput())
            extractImgFilter.SetComponents(1)
            extractImgFilter.Update()
            out = extractImgFilter.GetOutput()
        elif 'complex' in mode:
            out = ffti.GetOutput()
        else:
            colors.printc("Error in fft(): unknown mode", mode)
            raise RuntimeError()

        if center:
            center = vtk.vtkImageFourierCenter()
            center.SetInputData(out)
            center.Update()
            out = center.GetOutput()

        if 'complex' not in mode:
            if logscale:
                ils = vtk.vtkImageLogarithmicScale()
                ils.SetInputData(out)
                ils.SetConstant(logscale)
                ils.Update()
                out = ils.GetOutput()

        return Picture(out)

    def rfft(self, mode='magnitude'):
        """Reverse Fast Fourier transform of a picture."""

        ffti = vtk.vtkImageRFFT()
        ffti.SetInputData(self._data)
        ffti.Update()

        if 'mag' in mode:
            mag = vtk.vtkImageMagnitude()
            mag.SetInputData(ffti.GetOutput())
            mag.Update()
            out = mag.GetOutput()
        elif 'real' in mode:
            extractRealFilter = vtk.vtkImageExtractComponents()
            extractRealFilter.SetInputData(ffti.GetOutput())
            extractRealFilter.SetComponents(0)
            extractRealFilter.Update()
            out = extractRealFilter.GetOutput()
        elif 'imaginary' in mode:
            extractImgFilter = vtk.vtkImageExtractComponents()
            extractImgFilter.SetInputData(ffti.GetOutput())
            extractImgFilter.SetComponents(1)
            extractImgFilter.Update()
            out = extractImgFilter.GetOutput()
        elif 'complex' in mode:
            out = ffti.GetOutput()
        else:
            colors.printc("Error in rfft(): unknown mode", mode)
            raise RuntimeError()

        return Picture(out)

    def frequencyPassFilter(self, lowcutoff=None, highcutoff=None, order=3):
        """
        Low-pass and high-pass filtering become trivial in the frequency domain.
        A portion of the pixels/voxels are simply masked or attenuated.
        This function applies a high pass Butterworth filter that attenuates the
        frequency domain image with the function

        |G_Of_Omega|

        The gradual attenuation of the filter is important.
        A simple high-pass filter would simply mask a set of pixels in the frequency domain,
        but the abrupt transition would cause a ringing effect in the spatial domain.

        :param list lowcutoff:  the cutoff frequencies
        :param list highcutoff: the cutoff frequencies
        :param int order: order determines sharpness of the cutoff curve
        """
        #https://lorensen.github.io/VTKExamples/site/Cxx/ImageProcessing/IdealHighPass
        fft = vtk.vtkImageFFT()
        fft.SetInputData(self._data)
        fft.Update()
        out = fft.GetOutput()

        if highcutoff:
            butterworthLowPass = vtk.vtkImageButterworthLowPass()
            butterworthLowPass.SetInputData(out)
            butterworthLowPass.SetCutOff(highcutoff)
            butterworthLowPass.SetOrder(order)
            butterworthLowPass.Update()
            out = butterworthLowPass.GetOutput()

        if lowcutoff:
            butterworthHighPass = vtk.vtkImageButterworthHighPass()
            butterworthHighPass.SetInputData(out)
            butterworthHighPass.SetCutOff(lowcutoff)
            butterworthHighPass.SetOrder(order)
            butterworthHighPass.Update()
            out = butterworthHighPass.GetOutput()

        butterworthRfft = vtk.vtkImageRFFT()
        butterworthRfft.SetInputData(out)
        butterworthRfft.Update()

        butterworthReal = vtk.vtkImageExtractComponents()
        butterworthReal.SetInputData(butterworthRfft.GetOutput())
        butterworthReal.SetComponents(0)
        butterworthReal.Update()
        
        caster = vtk.vtkImageCast()
        caster. SetOutputScalarTypeToUnsignedChar()
        caster.SetInputData(butterworthReal.GetOutput())
        caster.Update()
        return self._update(caster.GetOutput())
    
    
    def blend(self, pic, alpha1=0.5, alpha2=0.5):
        """Take L, LA, RGB, or RGBA images as input and blends
        them according to the alpha values and/or the opacity setting for each input.
        """
        blf = vtk.vtkImageBlend()
        blf.AddInputData(self._data)
        blf.AddInputData(pic._data)
        blf.SetOpacity(0, alpha1)
        blf.SetOpacity(1, alpha2)
        blf.SetBlendModeToNormal()
        blf.Update()
        return self._update(blf.GetOutput())
    
    
    def warp(self, sourcePts=(), targetPts=(), transform=None, sigma=1, 
             mirroring=False, bc='w', alpha=1):
        """
        Warp an image using thin-plate splines.

        Parameters
        ----------
        sourcePts : list, optional
            source points. 
        targetPts : list, optional
            target points.
        transform : TYPE, optional
            a vtkTransform object can be supplied. The default is None.
        sigma : float, optional
            stiffness of the interpolation. The default is 1.
        mirroring : TYPE, optional
            fill the margins with a reflection of the original image. The default is False.
        bc : TYPE, optional
            fill the margins with a solid color. The default is 'w'.
        alpha : TYPE, optional
            opacity of the filled margins. The default is 1.
        """
        if transform is None:
            # source and target must be filled
            transform = vtk.vtkThinPlateSplineTransform()
            transform.SetBasisToR2LogR()
            if isinstance(sourcePts, vedo.Points):
                sourcePts = sourcePts.points()
            if isinstance(targetPts, vedo.Points):
                targetPts = targetPts.points()
    
            ns = len(sourcePts)
            nt = len(targetPts)
            if ns != nt:
                colors.printc("Error in picture.warp(): #source != #target points", ns, nt, c='r')
                raise RuntimeError()
                
            ptsou = vtk.vtkPoints()
            ptsou.SetNumberOfPoints(ns)   
    
            pttar = vtk.vtkPoints()
            pttar.SetNumberOfPoints(nt)
            
            for i in range(ns):
                p = sourcePts[i]
                ptsou.SetPoint(i, [p[0],p[1],0])
                p = targetPts[i]
                pttar.SetPoint(i, [p[0],p[1],0])

            transform.SetSigma(sigma)
            transform.SetSourceLandmarks(pttar)
            transform.SetTargetLandmarks(ptsou)
        else:
            # ignore source and target
            pass
        
        reslice = vtk.vtkImageReslice()
        reslice.SetInputData(self._data)
        reslice.SetOutputDimensionality(2)
        reslice.SetResliceTransform(transform)
        reslice.SetInterpolationModeToCubic()
        reslice.SetMirror(mirroring)
        c = np.array(colors.getColor(bc))*255
        reslice.SetBackgroundColor([c[0],c[1],c[2], alpha*255])
        reslice.Update()
        self.transform = transform
        return self._update(reslice.GetOutput())
    

    def threshold(self, value=None, flip=False):
        """
        Create a polygonal Mesh from a Picture by filling regions with pixels
        luminosity above a specified value.

        Parameters
        ----------
        value : float, optional
            The default is None, e.i. 1/3 of the scalar range.

        flip: bool, optional
            Flip polygon orientations

        Returns
        -------
        Mesh
            A polygonal mesh.
        """
        mgf = vtk.vtkImageMagnitude()
        mgf.SetInputData(self._data)
        mgf.Update()
        msq = vtk.vtkMarchingSquares()
        msq.SetInputData(mgf.GetOutput())
        if value is None:
            r0,r1 = self._data.GetScalarRange()
            value = r0 + (r1-r0)/3
        msq.SetValue(0, value)
        msq.Update()
        if flip:
            rs = vtk.vtkReverseSense()
            rs.SetInputData(msq.GetOutput())
            rs.ReverseCellsOn()
            rs.ReverseNormalsOff()
            rs.Update()
            output = rs.GetOutput()
        else:
            output = msq.GetOutput()
        ctr = vtk.vtkContourTriangulator()
        ctr.SetInputData(output)
        ctr.Update()
        return vedo.Mesh(ctr.GetOutput(), c='k').bc('t').lighting('off')


    def tomesh(self):
        """
        Convert an image to polygonal data (quads),
        with each polygon vertex assigned a RGBA value.
        """
        dims = self._data.GetDimensions()
        gr = vedo.shapes.Grid(sx=dims[0], sy=dims[1], resx=dims[0]-1, resy=dims[1]-1)
        gr.pos(int(dims[0]/2), int(dims[1]/2)).pickable(True).wireframe(False).lw(0)
        self._data.GetPointData().GetScalars().SetName("RGBA")
        gr.inputdata().GetPointData().AddArray(self._data.GetPointData().GetScalars())
        gr.inputdata().GetPointData().SetActiveScalars("RGBA")
        gr._mapper.SetArrayName("RGBA")
        gr._mapper.SetScalarModeToUsePointData()
        # gr._mapper.SetColorModeToDirectScalars()
        gr._mapper.ScalarVisibilityOn()
        gr.name = self.name
        gr.filename = self.filename
        return gr

    def tonumpy(self):
        """Get read-write access to pixels of a Picture object as a numpy array.
        Note that the shape is (nrofchannels, nx, ny).

        When you set values in the output image, you don't want numpy to reallocate the array
        but instead set values in the existing array, so use the [:] operator.
        Example: arr[:] = arr - 15

        If the array is modified call:
        ``picture.modified()``
        when all your modifications are completed.
        """
        nx, ny, _ = self._data.GetDimensions()
        nchan = self._data.GetPointData().GetScalars().GetNumberOfComponents()
        narray = utils.vtk2numpy(self._data.GetPointData().GetScalars()).reshape(ny,nx,nchan)
        narray = np.flip(narray, axis=0)
        return narray
    
    def modified(self):
        """Use in conjunction with ``tonumpy()`` to update any modifications to the picture array"""
        self._data.GetPointData().GetScalars().Modified()
        return self
    
    def write(self, filename):
        """Write picture to file as png or jpg."""
        vedo.io.write(self._data, filename)
        return self



